from django.db import models
from django.contrib.auth.models import User
# Create your models here.
class FilterAnalysis(models.Model):
    name = models.CharField(max_length=255)
    filterstring = models.TextField()
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)

class FamilyFilterAnalysis(models.Model):
    name = models.CharField(max_length=255)
    filterstring = models.TextField()
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)


class FilterConfig(models.Model):
    name = models.CharField(max_length=255)
    filterstring = models.TextField()
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)
    


from django.conf.urls import *
from django.contrib import admin
admin.autodiscover()
from django.conf import settings
from . import views
from django.urls import path

from .forms import ContactForm1, ContactForm2
from .views import ContactWizard

urlpatterns = [
    url(r'^$', views.index, name='analyses-index'),
    # url(r'^create/$', views.AnalysisCreate.as_view(), {}, 'analysis-create'),
    url(r'^create/$', views.create, {}, 'analysis-create'),
    url(r'detail/(?P<pk>\d+)/$', views.AnalysisDetailView.as_view(), name='analysis-detail'),
    url(r'delete/(?P<pk>\d+)/$', views.AnalysisDelete.as_view(), name='analysis-delete'),
    url(r'update/(?P<pk>\d+)/$', views.AnalysisUpdate.as_view(), name='analysis-update'),
    url(r'run/(?P<analysis_id>\d+)/$', views.run_analysis, name='analysis-run'),
    url(r'^contact/$', ContactWizard.as_view([ContactForm1, ContactForm2])),
]


from django import forms

from samples.models import SampleGroup

class ContactForm1(forms.Form):
    subject = forms.CharField(max_length=100)
    sender = forms.EmailField()

class ContactForm2(forms.Form):
    message = forms.CharField(widget=forms.Textarea)

ANALYSIS_TYPES = (
    ('qc', 'qc'),
    ('alignment', 'alignment'),
    ('variant calling', 'variant calling'),
    ('cnv identification', 'cnv identification'),
    ('pynnotator', 'pynnotator'),
)

PROVIDERS = (
    ('scw', 'Scaleway'),
    ('digital_ocean', 'Digital Ocean'),
    ('amazon', 'Amazon'),
    ('google', 'Google'),
)

class CreateAnalysis(forms.Form):
    name = forms.CharField(max_length=100)
    # description = forms.CharField(widget=forms.Textarea, required=False)
    # settings = forms.CharField(widget=forms.Textarea, required=False)
    providers = forms.MultipleChoiceField(
        required=False,
        choices=PROVIDERS,
    )

    analysis_types = forms.MultipleChoiceField(
        required=False,
        choices=ANALYSIS_TYPES,
    )
    files = forms.CharField(widget=forms.Textarea)
    # samplegroups = forms.ModelChoiceField(queryset = SampleGroup.objects.all() )



from django.db import models
from django.contrib.auth.models import User, Group
from individuals.models import Individual
from individuals.models import Group as IndividualGroup

import os
from datetime import datetime

from django.utils.text import slugify

class Case(models.Model):
    STATUS_CHOICES = (
        ('new', 'New'),
        ('solved', 'Solved'),
        ('closed', 'Closed'),
    )
    status = models.CharField(max_length=100,
                                      choices=STATUS_CHOICES,
                                      default='new')
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE)

    shared_with_users = models.ManyToManyField(User, related_name='shared_with', blank=True)
    shared_with_groups= models.ManyToManyField(Group, related_name='shared_with', blank=True)

    # individual = models.ForeignKey(Individual)
    name = models.CharField(max_length=600, blank=True)
    description = models.TextField(blank=True, null=True)
    # individuals = models.ManyToManyField(Individual)
   
    mother = models.ForeignKey(Individual, blank=True, related_name='mother', null=True, on_delete=models.CASCADE)
    father = models.ForeignKey(Individual, blank=True, related_name='father', null=True, on_delete=models.CASCADE)

    children = models.ManyToManyField(Individual, blank=True, related_name='children')

    cases = models.ManyToManyField(Individual, blank=True, related_name='cases')

    case_groups = models.ManyToManyField(Group, blank=True, related_name='case_groups', verbose_name="Groups of Cases")

    controls = models.ManyToManyField(Individual, blank=True, related_name='controls', verbose_name="Controls")

    control_groups = models.ManyToManyField(Group, blank=True, related_name='control_groups', verbose_name="Groups of Controls")
    

    # mother = models.ManyToManyField(Individual, related_name='mother')
    # children = models.ManyToManyField(Individual, related_name='children')


# class AnnotationCase(models.Model):
#   case=models.ForeignKey(Case)
#   annotation = models.TextField(blank=True)


    # 
    
    # featured = models.BooleanField(default=True)
    # variants_file = models.FileField(upload_to=get_upload_path, blank=True, help_text="File Format: VCF",max_length=600)
    
    # status = models.CharField(max_length=100, blank=True, editable=False)
    # n_records = models.IntegerField(blank=True, editable=False)
    # n_lines = models.IntegerField(blank=True, editable=False)
    
    # user = models.ForeignKey(User, editable=False)

    # creation_date = models.DateTimeField(auto_now_add=True,blank=True)
    # modified_date = models.DateTimeField(blank=True)

    # annotation_time = models.TextField(blank=True)
    # insertion_time = models.TextField(blank=True)

    # insertion_time_mongo = models.TextField(blank=True)

    # def __unicode__(self):
    #   return self.name

    # @models.permalink
    # def get_absolute_url(self):
    #   return ('individual-new',)

    # def __unicode__(self):
    #     return self.name

    # def save(self, *args, **kwargs):
    #     if not self.creation_date:
    #         self.creation_date = datetime.now()
    #     self.modified_date = datetime.now()
    #     return super(Individual, self).save(*args, **kwargs)


    # def delete(self, *args, **kwargs):
    #     super(Individual, self).delete(*args, **kwargs)

# class IndividualSummary(models.Model):
#     individual = models.ForeignKey(Individual)
#     ann_name = models.CharField(max_length=600)
#     ann_value = models.CharField(max_length=600)



# class IndividualMedicalConditionVariant(models.Model):
#     individual_variant = models.ForeignKey(Variant)
#     snp = models.ForeignKey(Snp)    
    
# class IndividualMedicalCondition(models.Model):
#     individual = models.ForeignKey(Individual)
#     medical_condition = models.ForeignKey(MedicalCondition)
#     variants = models.ManyToManyField(IndividualMedicalConditionVariant)
#     max_magnitude = models.FloatField(blank=True)
    
# class IndividualMedicineVariant(models.Model):
#     individual_variant = models.ForeignKey(Variant)
#     snp = models.ForeignKey(Snp)    
    
# class IndividualMedicine(models.Model):
#     individual = models.ForeignKey(Individual)
#     medicine = models.ForeignKey(Medicine)
#     variants = models.ManyToManyField(IndividualMedicineVariant)
#     max_magnitude = models.FloatField(blank=True)

# class IndividualHGMDMutation(models.Model):
#     individual = models.ForeignKey(Individual)
#     individual_variant = models.ForeignKey(Variant)
#     hgmdmutation = models.ForeignKey(HGMDMutation)


# class Group(models.Model):
#     name = models.CharField(max_length=128)
#     members = models.ManyToManyField(Individual)

#     def __unicode__(self):
#         return self.name

# class Relationship(models.Model):
#   group = models.ForeignKey(Group)
#   first_person = models.ForeignKey(Individual, related_name='first')
#   second_person = models.ForeignKey(Individual, related_name='second')
    
    
    



from django.conf import settings
from django.conf.urls import *

from django.contrib import admin
admin.autodiscover()

from . import views

urlpatterns = [
    url(r"^$", views.index, name="statistics_index"),    
]


# Create your views here.
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
# Imaginary function to handle an uploaded file.
#from somewhere import handle_uploaded_file

from variants.models import *
from individuals.models import *
from genes.models import Gene as Gene2
from diseases.models import *


import collections

@login_required
def index(request):

    statistics = collections.OrderedDict()

    statistics['Individuals'] = Individual.objects.all().count()
    statistics['Variants'] = Variant.objects.all().count()
    statistics['Genes'] = Gene2.objects.all().count()
    statistics['Diseases'] = Disease.objects.all().count()
    statistics['Diseases Genes'] = Gene.objects.all().count()
    # return render_to_response('statistics/index.html', context_instance=RequestContext(request))
    return render(request, 'statistics/index.html', {"statistics": statistics})



# -*- coding: utf-8 -*-

# Create your tasks here
from __future__ import absolute_import, unicode_literals
from celery import shared_task

# from celery.task import Task
# from celery.registry import tasks
from django.db import transaction
from django.core.files import File
# from celery import task

from individuals.models import *

from variants.models import *

# from mysql_bulk_insert import bulk_insert
from django.shortcuts import render, get_object_or_404
import os
import datetime
from django.core.mail import send_mail
# from snpedia.models import *
from diseases.models import HGMDMutation
from django.conf import settings

import zipfile
import gzip
import pickle
import tarfile
from collections import OrderedDict


import json
import vcf

from datetime import timedelta
from django.template.defaultfilters import slugify


@shared_task()
def test():
    print("Running periodic task!")
    individuals = Individual.objects.filter(user=None)
    for individual in individuals:
        time_difference = datetime.datetime.now()-individual.creation_date
        if time_difference.days > 0:
            #delete individuals
            os.system('rm -rf %s/genomes/public/%s' % (settings.BASE_DIR, individual_id))
            individual.delete()




# Generated by Django 2.0.1 on 2018-02-12 19:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('individuals', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Variant',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('index', models.TextField(db_index=True)),
                ('pos_index', models.TextField(db_index=True)),
                ('chr', models.TextField(db_index=True, verbose_name='Chr')),
                ('pos', models.IntegerField(db_index=True)),
                ('variant_id', models.TextField(db_index=True, verbose_name='ID')),
                ('ref', models.TextField(blank=True, db_index=True, null=True)),
                ('alt', models.TextField(blank=True, db_index=True, null=True)),
                ('qual', models.FloatField(db_index=True)),
                ('filter', models.TextField(db_index=True)),
                ('info', models.TextField(blank=True, null=True)),
                ('format', models.TextField(blank=True, db_index=True, null=True)),
                ('genotype_col', models.TextField(blank=True, db_index=True, null=True)),
                ('genotype', models.TextField(db_index=True)),
                ('read_depth', models.IntegerField()),
                ('gene', models.TextField(blank=True, db_index=True, null=True)),
                ('mutation_type', models.TextField(db_index=True, null=True)),
                ('vartype', models.TextField(db_index=True, null=True)),
                ('genomes1k_maf', models.FloatField(blank=True, db_index=True, null=True, verbose_name='1000 Genomes Frequency')),
                ('dbsnp_maf', models.FloatField(blank=True, db_index=True, null=True, verbose_name='dbSNP Frequency')),
                ('esp_maf', models.FloatField(blank=True, db_index=True, null=True, verbose_name='ESP6500 Frequency')),
                ('dbsnp_build', models.IntegerField(db_index=True, null=True)),
                ('sift', models.FloatField(blank=True, db_index=True, null=True)),
                ('sift_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('polyphen2', models.FloatField(blank=True, db_index=True, null=True)),
                ('polyphen2_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('condel', models.FloatField(blank=True, db_index=True, null=True)),
                ('condel_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('dann', models.FloatField(blank=True, db_index=True, null=True)),
                ('cadd', models.FloatField(blank=True, db_index=True, null=True)),
                ('hi_index_str', models.TextField(blank=True, db_index=True, null=True)),
                ('hi_index', models.FloatField(blank=True, db_index=True, null=True)),
                ('hi_index_perc', models.FloatField(blank=True, db_index=True, null=True)),
                ('is_at_omim', models.BooleanField(db_index=True, default=False)),
                ('is_at_hgmd', models.BooleanField(db_index=True, default=False)),
                ('hgmd_entries', models.TextField(blank=True, db_index=True, null=True)),
                ('snpeff_effect', models.TextField(blank=True, db_index=True, null=True)),
                ('snpeff_impact', models.TextField(blank=True, db_index=True, null=True)),
                ('snpeff_gene_name', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_allele', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_gene', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_feature', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_feature_type', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_consequence', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_cdna_position', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_cds_position', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_protein_position', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_amino_acids', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_codons', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_existing_variation', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_distance', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_strand', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_symbol', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_symbol_source', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_sift', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_polyphen', models.TextField(blank=True, db_index=True, null=True)),
                ('vep_condel', models.TextField(blank=True, db_index=True, null=True)),
                ('ensembl_clin_HGMD', models.BooleanField(db_index=True, default=False)),
                ('clinvar_CLNSRC', models.TextField(blank=True, db_index=True, null=True)),
                ('SIFT_score', models.TextField(blank=True, db_index=True, null=True)),
                ('SIFT_converted_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('Uniprot_acc_Polyphen2', models.TextField(blank=True, db_index=True, null=True)),
                ('Uniprot_id_Polyphen2', models.TextField(blank=True, db_index=True, null=True)),
                ('Uniprot_aapos_Polyphen2', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HDIV_score', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HDIV_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HDIV_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HVAR_score', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HVAR_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('Polyphen2_HVAR_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('LRT_score', models.TextField(blank=True, db_index=True, null=True)),
                ('LRT_converted_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('LRT_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('LRT_Omega', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationTaster_score', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationTaster_converted_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationTaster_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationTaster_model', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationTaster_AAE', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationAssessor_UniprotID', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationAssessor_variant', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationAssessor_score', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationAssessor_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('MutationAssessor_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('FATHMM_score', models.TextField(blank=True, db_index=True, null=True)),
                ('FATHMM_converted_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('FATHMM_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('PROVEAN_score', models.TextField(blank=True, db_index=True, null=True)),
                ('PROVEAN_converted_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('PROVEAN_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('Transcript_id_VEST3', models.TextField(blank=True, db_index=True, null=True)),
                ('Transcript_var_VEST3', models.TextField(blank=True, db_index=True, null=True)),
                ('VEST3_score', models.TextField(blank=True, db_index=True, null=True)),
                ('VEST3_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaSVM_score', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaSVM_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaSVM_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaLR_score', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaLR_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('MetaLR_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('Reliability_index', models.TextField(blank=True, db_index=True, null=True)),
                ('CADD_raw', models.TextField(blank=True, db_index=True, null=True)),
                ('CADD_raw_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('CADD_phred', models.TextField(blank=True, db_index=True, null=True)),
                ('DANN_score', models.TextField(blank=True, db_index=True, null=True)),
                ('DANN_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('fathmm_MKL_coding_score', models.TextField(blank=True, db_index=True, null=True)),
                ('fathmm_MKL_coding_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('fathmm_MKL_coding_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('fathmm_MKL_coding_group', models.TextField(blank=True, db_index=True, null=True)),
                ('Eigen_raw', models.TextField(blank=True, db_index=True, null=True)),
                ('Eigen_phred', models.TextField(blank=True, db_index=True, null=True)),
                ('Eigen_raw_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('Eigen_PC_raw', models.TextField(blank=True, db_index=True, null=True)),
                ('Eigen_PC_raw_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('GenoCanyon_score', models.TextField(blank=True, db_index=True, null=True)),
                ('GenoCanyon_score_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('integrated_fitCons_score', models.TextField(blank=True, db_index=True, null=True)),
                ('integrated_fitCons_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('integrated_confidence_value', models.TextField(blank=True, db_index=True, null=True)),
                ('GM12878_fitCons_score', models.TextField(blank=True, db_index=True, null=True)),
                ('GM12878_fitCons_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('GM12878_confidence_value', models.TextField(blank=True, db_index=True, null=True)),
                ('H1_hESC_fitCons_score', models.TextField(blank=True, db_index=True, null=True)),
                ('H1_hESC_fitCons_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('H1_hESC_confidence_value', models.TextField(blank=True, db_index=True, null=True)),
                ('HUVEC_fitCons_score', models.TextField(blank=True, db_index=True, null=True)),
                ('HUVEC_fitCons_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('HUVEC_confidence_value', models.TextField(blank=True, db_index=True, null=True)),
                ('GERP_NR', models.TextField(blank=True, db_index=True, null=True)),
                ('GERP_RS', models.TextField(blank=True, db_index=True, null=True)),
                ('GERP_RS_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('phyloP100way_vertebrate', models.TextField(blank=True, db_index=True, null=True)),
                ('phyloP100way_vertebrate_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('phyloP20way_mammalian', models.TextField(blank=True, db_index=True, null=True)),
                ('phyloP20way_mammalian_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('phastCons100way_vertebrate', models.TextField(blank=True, db_index=True, null=True)),
                ('phastCons100way_vertebrate_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('phastCons20way_mammalian', models.TextField(blank=True, db_index=True, null=True)),
                ('phastCons20way_mammalian_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('SiPhy_29way_pi', models.TextField(blank=True, db_index=True, null=True)),
                ('SiPhy_29way_logOdds', models.TextField(blank=True, db_index=True, null=True)),
                ('SiPhy_29way_logOdds_rankscore', models.TextField(blank=True, db_index=True, null=True)),
                ('clinvar_rs', models.TextField(blank=True, db_index=True, null=True)),
                ('clinvar_clnsig', models.TextField(blank=True, db_index=True, null=True)),
                ('clinvar_trait', models.TextField(blank=True, db_index=True, null=True)),
                ('clinvar_golden_stars', models.TextField(blank=True, db_index=True, null=True)),
                ('mcap_score', models.FloatField(blank=True, db_index=True, null=True)),
                ('mcap_rankscore', models.FloatField(blank=True, db_index=True, null=True)),
                ('mcap_pred', models.TextField(blank=True, db_index=True, null=True)),
                ('revel_score', models.TextField(blank=True, db_index=True, null=True)),
                ('individual', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='individuals.Individual')),
            ],
        ),
    ]



from django.conf.urls import *

from django.contrib import admin

from django.views.generic import TemplateView

admin.autodiscover()

from django.conf import settings

from django.conf.urls.static import static

from . import views

urlpatterns = [
    # Examples:
    # url(r'^$', 'mendelmd.views.home', name='home'),
    # url(r'^blog/', include('blog.urls')),
    url(r'^$', views.upload, name='upload'),

] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)




from django.forms import ModelForm
from cases.models import Case
from django import forms
from individuals.models import Individual

# Create the form class.
class CaseForm(ModelForm):
    class Meta:
        model = Case
        fields = '__all__'


from django.contrib import admin
from django.conf.urls import url, include
from django.contrib.auth.models import User

from . import views

urlpatterns = [

    url(r'^$', views.index, name='sample_index'),
    url(r'^create_group/$', views.create_group, name='create_group'),
    url(r'^groups/$', views.SampleGroupList.as_view(), name='samplegroup-list'),
    url(r'^groups/delete/(?P<pk>\d+)$', views.SampleGroupDelete.as_view(), name='samplegroup_delete'),
    url(r'^groups/(?P<pk>\d+)/$', views.group_detail, name='samplegroup-view'),
    url(r'^groups/edit/(?P<pk>\d+)$', views.SampleGroupUpdateView.as_view(), name='samplegroup-edit'),
    url(r'^import_vcf/(?P<pk>\d+)/$', views.sample_import_vcf, name='sample_import_vcf'),
    # url(r'^delete_group/(?P<pk>\d+)$', SampleGroupDeleteView.as_view(), {}, 'group_delete'),
    url(r'^view/(?P<pk>\d+)$', views.SampleDetailView.as_view(), name='sample_view'),
    url(r'^bulk_action/$', views.bulk_action, name='samples-bulk-action'),
    
    # url(r'^search/$', views.search, name='sample_search'),
    # url(r'^action/$', views.action, name='sample_action'),

    # url(r'^edit/(?P<pk>\d+)$', views.sample_update, name='sample_edit'),
    # url(r'^create_analysis/(?P<pk>\d+)$', views.create_analysis, name='sample_create_analysis'),
    # url(r'^delete/(?P<pk>\d+)$', views.sample_delete, name='sample_delete'),

   #  url(r'^new$', views.ServerCreate.as_view(), name='server_new'),
  	# url(r'^edit/(?P<pk>\d+)$', views.ServerUpdate.as_view(), name='server_edit'),
  	# url(r'^delete/(?P<pk>\d+)$', views.ServerDelete.as_view(), name='server_delete'),

    # url(r'^upload/', views.upload, name='upload'),

]



from django.db import models
from django.conf import settings
from django.contrib.auth.models import User, Group
from django.template.defaultfilters import slugify
from django.contrib.postgres.fields import JSONField

# Create your models here.
class File(models.Model):

    def get_upload_path(self, filename):
        if self.user != None:
            string = "%s/media/%s/%s/%s" % (settings.BASE_DIR, slugify(self.user.username), self.id, filename)
        else:
            string = "%s/media/public/%s/%s" % (settings.BASE_DIR, self.id, filename)
        return string

    def get_absolute_url(self):
        return "/files/"

    def __str__(self):
        return self.name

    user = models.ForeignKey(User, editable=False, null=True, on_delete=models.CASCADE)
    
    name = models.CharField(max_length=600, blank=True)
    
    size = models.BigIntegerField(null=True, blank=True)
    
    file_type = models.TextField(null=True, blank=True)
    extension = models.TextField(null=True, blank=True)

    location = models.TextField(null=True, blank=True)    
    local_file = models.FileField(upload_to=get_upload_path, blank=True, help_text="File Format: VCF",max_length=600)
    remote_location = models.TextField(null=True, blank=True)
    url = models.TextField(null=True, blank=True)
    
    status = models.TextField(null=True, blank=True)
    last_output = models.TextField(null=True, blank=True)
    params = JSONField(null=True, blank=True)

    md5 = models.TextField(null=True, blank=True)

    last_modified = models.DateTimeField(null=True, blank=True)

    creation_date = models.DateTimeField(auto_now_add=True,null=True, blank=True)
    modified_date = models.DateTimeField(auto_now=True, null=True, blank=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'postgres',
        'USER': 'postgres',
        'HOST': 'db',
        'PORT': 5432,
    }
}

SECRET_KEY = '*efl#$$!@93)8397wwf8hy3873&ad8h7d2w-JKFCGYURaonyGUimaraesCorreayus5mzcx&@'
DEBUG = True
#EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
#EMAIL_HOST = 'smtp.gmail.com'
#EMAIL_HOST_USER = ''
#EMAIL_HOST_PASSWORD = ''
#EMAIL_PORT = 587
#EMAIL_USE_TLS = True
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
# STATICFILES_DIRS = (
#     os.path.join(BASE_DIR, "static"),
#    '/var/www/html/static/',
# )

STATIC_URL = '/var/www/static/'
STATICFILES_DIRS = [
    os.path.join(BASE_DIR, "static"),
]



#!/usr/bin/env python
import os
import sys

sys.path.insert(0, os.path.join(os.path.dirname(__file__), "apps"))

if __name__ == "__main__":
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "mendelmd.settings")

    # Add the apps directoriy to Python's path. In production it will
    # be necessary to add the apps directory to the path, too.
    # from os.path import abspath, dirname, join
    # PROJECT_ROOT = abspath(dirname(__file__))
    # sys.path.append(join(PROJECT_ROOT, "apps"))
    

    from django.core.management import execute_from_command_line

    execute_from_command_line(sys.argv)



from django.db import models
from django.contrib.auth.models import User
from django.contrib.postgres.fields import JSONField

class S3Credential(models.Model):
    def get_absolute_url(self):
        return "/settings/"
    user = models.ForeignKey(User, on_delete=models.CASCADE, editable=False)
    name = models.CharField(max_length=255)
    access_key = models.CharField(max_length=255)
    secret_key = models.CharField(max_length=255)
    buckets = models.TextField(null=True, blank=True)
    exclude_paths= models.TextField(null=True, blank=True)
    exclude_files = models.TextField(null=True, blank=True)

class Provider(models.Model):
    def get_absolute_url(self):
        return "/settings/"
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    name = models.CharField(max_length=255)
    type = models.CharField(max_length=255)
    config = JSONField()
    def __str__(self):
        return self.name

class Subnet(models.Model):
    provider = models.ForeignKey(Provider, on_delete=models.CASCADE, editable=False)
    name = models.CharField(max_length=255)
    config = JSONField()
    count = models.IntegerField(null=True, blank=True)

class Profile(models.Model):
    user = models.OneToOneField(
            User,
            on_delete=models.CASCADE,
            primary_key=True,
        )
    params = JSONField()


from django.shortcuts import render
from django.shortcuts import redirect

def index(request):
	# print "Hello"
	#if user is logged redirect
	if request.user.is_authenticated:
		return redirect('dashboard')

	return render(request, 'pages/index.html')

def docs(request):
    return redirect("https://mendelmd.readthedocs.io/")


from django.conf.urls import url

from . import views

urlpatterns = [
    url(r'^$', views.index, name='files-index'),
    url(r'^import_files/', views.import_files, name='files-import'),
    url(r'^upload/', views.upload, name='upload'),
    url(r'^view/(?P<file_id>[0-9]+)/$', views.view, name='file-view'),
    url(r'^update/(?P<pk>[0-9]+)/$', views.FileUpdate.as_view(), name='file-update'),
    url(r'^delete/(?P<pk>[0-9]+)/$', views.FileDelete.as_view(), name='file-delete'),
    url(r'^run_task/(?P<pk>[0-9]+)/$', views.run_task, name='files-runtask'),
    url(r'^bulk_action/$', views.bulk_action, name='files-bulk-action'),
    url(r'^run_task/$', views.run_task, name='files-runtask'),
]



# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('individuals', '__first__'),
        ('auth', '0009_alter_user_last_name_max_length'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Case',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('new', 'New'), ('solved', 'Solved'), ('closed', 'Closed')], default='new', max_length=100)),
                ('name', models.CharField(blank=True, max_length=600)),
                ('description', models.TextField(blank=True, null=True)),
                ('case_groups', models.ManyToManyField(blank=True, related_name='case_groups', to='auth.Group', verbose_name='Groups of Cases')),
                ('cases', models.ManyToManyField(blank=True, related_name='cases', to='individuals.Individual')),
                ('children', models.ManyToManyField(blank=True, related_name='children', to='individuals.Individual')),
                ('control_groups', models.ManyToManyField(blank=True, related_name='control_groups', to='auth.Group', verbose_name='Groups of Controls')),
                ('controls', models.ManyToManyField(blank=True, related_name='controls', to='individuals.Individual', verbose_name='Controls')),
                ('father', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='father', to='individuals.Individual')),
                ('mother', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='mother', to='individuals.Individual')),
                ('shared_with_groups', models.ManyToManyField(blank=True, related_name='shared_with', to='auth.Group')),
                ('shared_with_users', models.ManyToManyField(blank=True, related_name='shared_with', to=settings.AUTH_USER_MODEL)),
                ('user', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.http import HttpResponseRedirect, HttpResponse
from django.shortcuts import redirect, get_object_or_404, render
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.db.models import Q
from django.db.models import Count, Sum, Avg, Min, Max
from django.core import serializers
from django.views.generic import UpdateView, DeleteView
from django.urls import reverse

#from django.contrib.formtools.wizard import FormWizard
# from django.contrib.formtools.wizard.views import SessionWizardView

from .forms import FilterWiZardForm1, FilterWiZardForm2, FilterWiZardForm3, FamilyAnalysisForm
from datetime import datetime
import operator
from django.conf import settings
from .forms import *
import csv
import pickle

from individuals.models import *
from filter_analysis.forms import *
from filter_analysis.models import *
from genes.models import GeneGroup, Gene, CGDEntry, GeneList

from .filter_options import *

from diseases.models import Disease, HGMDGene, HGMDPhenotype
from diseases.models import Gene as GeneDisease

import json

from django.shortcuts import render

# import django_tables2 as tables
# from django_tables2 import RequestConfig

#a method to integrate all different filters
#should return an object with all the results

def test(request):
    form = FilterAnalysisForm(request.user)

    return render(request, 'filter_analysis/test.html', {'form':form});


def oneclick(request):
    form = FilterAnalysisForm(request.user)
    if request.method == 'GET':
        # print request.META
        query_string = request.META['QUERY_STRING']
        # print(query_string)
        if query_string != '':
            new_query_string = []
            query_string = query_string.split('&')
            for item in query_string:
                if not (item.startswith('csrfmiddlewaretoken') or item.startswith('hash') or item.startswith('wizard')):
                    #get here only the ones need to be cleaned Ex. 1-chr
                    #item = "-".join(item.split('-', 2)[1:])
                    new_query_string.append(item)

            #create new query
            filterstring = "&".join(new_query_string)
            return redirect(reverse('filter_analysis')+'?'+filterstring)

    return render(request, 'filter_analysis/oneclick.html', {'form':form});


def calculate_summary(step, args, query, exclude):

    print('enter calculate summary')    
    summary = {}
    summary[step] = {}

    # 1) calculate number of variants
    variants = Variant.objects.filter(*args, **query).exclude(**exclude)
    n_variants = variants.count()#.prefetch_related('individual')
    summary[step]['variants'] = n_variants
    
    #2)Calculate number of genes
    genes = variants.values_list('gene', flat=True)
    genes = sorted(list(set(genes)))

    summary[step]['genes'] = len(genes)

    #3)Calculate number of genes at omim
    queries = [Q(genes__icontains=value) for value in list(genes)]
    if len(queries) > 0:
        query = queries.pop()
        for item in queries:
            query |= item
        summary[step]['genes_at_omim'] = GeneDisease.objects.filter(official_name__in=list(genes)).count()#query
    else:
        summary[step]['genes_at_omim'] = '-'
    

    print('exit calculate summary')

    return summary

#show all steps from filter analysis 
def filter_analysis_table(request):
    print('dynamic filter analysis')
    results = {}
    
    query = {}
    exclude = {}
    summary = {}
    args = []

    table_list = []

    form = FilterAnalysisForm(request.user, request.GET)

    filter_by_individuals(request, args, query, exclude)

    table_list.append(calculate_summary('initial', args, query, exclude))

    #CHR
    filter_chr(request, query)
    #pos
    filter_pos(request, query)
    #snp_list
    filter_snp_list(request, query)
    #Gene List, this is entered by hand
    filter_gene_list(request, query, args)
    #Inheritance_model
    filter_mutation_type(request, args)

    table_list.append(calculate_summary('mutation_type', args, query, exclude))

    #Clinical
    filter_cln(request, query)
    #Variant type SNP EFF       
    filter_variant_type_snpeff(request, query)
    #DBSNP
    filter_dbsnp(request, query)
    #1000genomes
    filter_by_1000g(request, args)
    #dbsnp Freq
    filter_by_dbsnp(request, args)
    #Exome Variation Server Freq
    filter_by_esp(request, args)
    #hi score
    # filter_by_hi_score(request, args)
    #SIFT
    filter_by_sift(request, args)
    #POLYPHEN
    filter_by_pp2(request, args)
    filter_by_cadd(request, args)
    filter_by_mcap(request, args)
    filter_by_rf_score(request, args)
    filter_by_ada_score(request, args)

    filter_by_segdup(request, args)
    #filter by disease databases
    filter_omim(request, args)
    filter_cgd(request, args)
    filter_hgmd(request, args)
    filter_genelists(request, query, args, exclude)
    #DBSNP Build
    filter_dbsnp_build(request, args)            
    #Read Depth
    filter_read_depth(request, args)            
    filter_qual(request, args)
    filter_filter(request, query)
    #Functional class
    filter_func_class(request, query)
    #IMPACT
    filter_impact(request, query)
    
    # filter_by_individuals_full(request, args, query, exclude)

    #last call to the DATABASE Finally!!!!!! 
    variants = Variant.objects.filter(*args, **query).exclude(**exclude).prefetch_related('individual')

    table_list.append(calculate_summary('final', args, query, exclude))

    #summary
    summary['genes'] = variants.values_list('gene', flat=True)
    summary['genes'] = sorted(list(set(summary['genes'])))
    summary['n_genes'] =  len(summary['genes'])
    summary['has_variants'] = True
    
    #show summary of genes associated with diseases every time
    #if summary['n_genes'] < 500:

    genes = Gene.objects.filter(symbol__in=list(summary['genes'])).values('symbol', 'diseases').prefetch_related('diseases')                
    genes_hgmd = HGMDGene.objects.filter(symbol__in=list(summary['genes'])).prefetch_related('diseases')# | reduce(lambda x, y: x | y, [Q(aliases__icontains=word) for word in list(summary['genes'])]))
    #return Disease.objects.filter(Q(name__icontains=name)|Q(genes__icontains=name))
    #for each gene name create a query
    queries = [Q(genes__icontains=value) for value in list(summary['genes'])]
    if len(queries) > 0:
        query = queries.pop()
        for item in queries:
            query |= item
        genes_omim = GeneDisease.objects.filter(official_name__in=list(summary['genes'])).prefetch_related('diseases').order_by('official_name')#query
    else:
        genes_omim = []
    #get genes at CGD
    genes_cgd = CGDEntry.objects.filter(GENE__in=list(summary['genes'])).prefetch_related('CONDITIONS')

    # else:
    #     genes = []
    #     genes_hgmd = []
    #     genes_omim = []
    #     genes_cgd = []

    summary['n_variants'] = len(variants)

    # 'variants':variants, 
    #     'form':form, 
    #     'summary':summary, 
    #     'query_string':query_string, 
    #     'filteranalysis':filteranalysis, 
    #     'filterconfigs':filterconfigs, 
    #     'genes':genes, 
    #     'genes_hgmd':genes_hgmd,
    #     'genes_omim':genes_omim,
    #     'genes_cgd':genes_cgd
    #return results

    table = table_list

    return render(request, 'filter_analysis/table.html', 
        {'table':table})



def filter_analysis(request,query, args, exclude):
    '''
    This function receives request and returns a dictionary with the variants
    '''
    # print('filter analysis')

    #CHR
    filter_chr(request, query)
    #pos
    filter_pos(request, query)
    #snp_list
    filter_snp_list(request, query, exclude)

    #Gene List, this is entered by hand
    filter_gene_list(request, query, args)
    #Mutation Type
    filter_mutation_type(request, args)
    #Variant type SNP EFF       
    filter_effect(request, query)
    #Exclude variants at dbsnp
    filter_dbsnp(request, query)

    filter_varisnp(request, query, exclude)
    # print 'exclude debug1', exclude

    #1000genomes
    filter_by_1000g(request, args)
    #dbsnp Freq
    filter_by_dbsnp(request, args)
    #Exome Variation Server Freq
    filter_by_esp(request, args)

    #SIFT
    filter_by_sift(request, args)
    #POLYPHEN
    filter_by_pp2(request, args)
    filter_by_cadd(request, args)
    filter_by_mcap(request, args)
    filter_by_rf_score(request, args)
    filter_by_ada_score(request, args)
    
    #filter by disease databases
    filter_omim(request, args)
    filter_cgd(request, args)
    filter_hgmd(request, args)
    filter_genelists(request, query, args, exclude)
    #DBSNP Build
    filter_dbsnp_build(request, args)            
    #Read Depth
    filter_read_depth(request, args)            
    filter_qual(request, args)
    filter_filter(request, query)
    #Functional class
    filter_func_class(request, query)
    #IMPACT
    filter_impact(request, query)
    filter_is_at_hgmd(request, query)
    filter_clnsig(request, query)

    # filter_individuals(request, query)
    
    # filter_by_individuals(request, *query, *args, *exclude)

def get_genes(summary):

    genes ={}

    genes['genes'] = Gene.objects.filter(symbol__in=list(summary['genes'])).values('symbol', 'diseases').prefetch_related('diseases')                
    
    genes['genes_hgmd'] = HGMDGene.objects.filter(symbol__in=list(summary['genes'])).prefetch_related('diseases')

    #for each gene name create a query
    queries = [Q(genes__icontains=value) for value in list(summary['genes'])]
    if len(queries) > 0:
        query = queries.pop()
        for item in queries:
            query |= item
        genes['genes_omim'] = GeneDisease.objects.filter(official_name__in=list(summary['genes'])).prefetch_related('diseases').order_by('official_name')#query
    else:
        genes['genes_omim'] = []
    #get genes at CGD
    genes['genes_cgd'] = CGDEntry.objects.filter(GENE__in=list(summary['genes'])).prefetch_related('CONDITIONS').order_by('GENE')

    return genes

#This is the most important function
def index(request):
    query = {}
    exclude = {}
    summary = {}
    args = []

    #this removes page from QUERY_STRING
    query_string = request.META['QUERY_STRING']
    new_query_string = []
    for item in query_string.split('&'):
        if not item.startswith('page'):
             new_query_string.append(item)
    query_string = "&".join(new_query_string)
    
    # print 'query string'
    # print query_string 
    
    filteranalysis = FilterAnalysis.objects.all().prefetch_related('user')
    filterconfigs = FilterConfig.objects.all().prefetch_related('user')
    
    #this is for checking if query is empty (to avoid loading at the first time)
    if query_string != [''] and query_string != '' :
        
        #make it GET for explicit urls
        if request.method == 'GET':

            form = FilterAnalysisForm(request.user, request.GET)

            if 'sort_by' in request.GET:
                sort_by = request.GET['sort_by']
                if sort_by == 'desc':
                    sort = 'asc'
                else:
                    sort = 'desc'
            else:
                sort = 'asc'

            if 'order_by' in request.GET:
                if sort_by == 'desc':
                    order_by = '-%s' % request.GET['order_by']
                else:
                    order_by = request.GET['order_by']
            else:
                order_by = 'gene'

            filter_analysis(request, query, args, exclude)

            individuals_list = filter_individuals_variants(request, query, args, exclude)

            if len(individuals_list) > 0:
                #only add to query after filtering the indexes 
                query['individual_id__in'] = individuals_list

                filter_variants_per_gene(request, query, args, exclude)
                filter_genes_in_common(request, query, args, exclude)
                filter_positions_in_common(request, query, args, exclude)
            
            # print('exclude keys', list(exclude.keys()))
            #last call to the DATABASE Finally!!!!!! 
            variants = Variant.objects.filter(*args, **query).exclude(**exclude).prefetch_related('individual').order_by(order_by)
            
            #export_to_csv(request, variants)
            export = request.GET.get('export', '')
            if export != '':
                if export == 'csv':
                    response = HttpResponse(content_type='text/csv')
                    response['Content-Disposition'] = 'attachment; filename=export.csv'
                    writer = csv.writer(response)
                    
                elif export == 'txt':
                    response = HttpResponse(content_type='text/plain')
                    response['Content-Disposition'] = 'attachment; filename=export.txt'
                    writer = csv.writer(response, delimiter='\t', quoting=csv.QUOTE_NONE)    
                writer.writerow(['Individual', 'Index', 'Pos_index', 'Chr', 'Pos', 'Variant_id', 'Ref', 'Alt', 'Qual', 'Filter', 'Info', 'Format', 'Genotype_col', 'Genotype', 'Read_depth', 'Gene', 'Mutation_type', 'Vartype', 'Genomes1k_maf', 'Dbsnp_maf', 'Esp_maf', 'Dbsnp_build', 'Sift', 'Sift_pred', 'Polyphen2', 'Polyphen2_pred', 'Condel', 'Condel_pred', 'DANN', 'CADD', 'Is_at_omim', 'Is_at_hgmd', 'Hgmd_entries', 'Effect', 'Impact', 'Func_class', 'Codon_change', 'Aa_change', 'Gene_name', 'Biotype', 'Gene_coding', 'Transcript_id', 'Exon_rank', 'Allele', 'Gene', 'Feature', 'Feature_type', 'Consequence', 'Cdna_position', 'Cds_position', 'Protein_position', 'Amino_acids', 'Codons', 'Existing_variation', 'Distance', 'Strand', 'Symbol', 'Symbol_source', 'Sift', 'Polyphen', 'Condel']) 
                for variant in variants:
                    # print 'variant', variant.index
                    writer.writerow([variant.individual, variant.index, variant.pos_index, variant.chr, variant.pos, variant.variant_id, variant.ref, variant.alt, variant.qual, variant.filter, json.loads(variant.info), variant.format, variant.genotype_col, variant.genotype, variant.read_depth, variant.gene, variant.mutation_type, variant.vartype, variant.genomes1k_maf, variant.dbsnp_maf, variant.esp_maf, variant.dbsnp_build, variant.sift, variant.sift_pred, variant.polyphen2, variant.polyphen2_pred, variant.condel, variant.condel_pred, variant.dann, variant.cadd, variant.is_at_omim, variant.is_at_hgmd, variant.hgmd_entries, variant.snpeff_effect, variant.snpeff_impact, variant.snpeff_func_class, variant.snpeff_codon_change, variant.snpeff_aa_change, variant.snpeff_gene_name, variant.snpeff_biotype, variant.snpeff_gene_coding, variant.snpeff_transcript_id, variant.snpeff_exon_rank, variant.vep_allele, variant.vep_gene, variant.vep_feature, variant.vep_feature_type, variant.vep_consequence, variant.vep_cdna_position, variant.vep_cds_position, variant.vep_protein_position, variant.vep_amino_acids, variant.vep_codons, variant.vep_existing_variation, variant.vep_distance, variant.vep_strand, variant.vep_symbol, variant.vep_symbol_source, variant.vep_sift, variant.vep_polyphen, variant.vep_condel])
                return response

            #summary
            summary['genes'] = variants.values_list('gene', flat=True)
            # print(summary['genes'])
            summary['genes'] = list(set(summary['genes']))
            summary['genes'] = sorted(list(filter(None.__ne__, summary['genes'])))
            summary['n_genes'] =  len(summary['genes'])
            summary['has_variants'] = True
            summary['n_variants'] = len(variants)

            genes = get_genes(summary)            

            paginator = Paginator(variants, 100) # Show 25 contacts per page
            try:
               page = int(request.GET.get('page', '1'))
            except ValueError:
               page = 1
            try:
               variants = paginator.page(page)
            except PageNotAnInteger:
               # If page is not an integer, deliver first page.
               variants = paginator.page(1)
            except EmptyPage:
               # If page is out of range (e.g. 9999), deliver last page of results.
               variants = paginator.page(paginator.num_pages)
    #first entrance in methods
    else:
        variants = []
        summary = {}
        genes = {}

        summary['has_variants'] = False
        #fake number just not to open accordion on view 
        summary['n_genes'] = 1000
        
        genes['genes'] = []
        genes['genes_hgmd'] = []
        genes['genes_omim'] = []
        genes['genes_cgd'] = []
        form = FilterAnalysisForm(request.user)



    # print(summary)
    return render(request, 'filteranalysis/index.html', 
        {'variants':variants, 
        'form':form, 
        'summary':summary,
        'query_string':query_string, 
        'filteranalysis':filteranalysis, 
        'filterconfigs':filterconfigs, 
        'genes':genes['genes'], 
        'genes_hgmd':genes['genes_hgmd'],
        'genes_omim':genes['genes_omim'],
        'genes_cgd':genes['genes_cgd']})


def filter_family_analysis(request, query, args, exclude):

    print('Get family variants')
    father = request.GET.get('father', '')
    mother = request.GET.get('mother', '')
    

    parents_variants = {'father':{}, 'mother':{}}
    parents_list = []
    parents = {}
    if father != '':
        parents['father']=father
        parents_list.append(father)
    if mother != '':
        parents['mother']=mother
        parents_list.append(mother)
    # print 'parents', parents            
        
    # parents = {'father':father, 'mother':mother}
    # query_bkp = query['individual_id__in']
    #for each parent get variants from parents and build dict for future filtering
    parents_indexes = Variant.objects.filter(individual__id__in=parents_list).values_list('index', flat=True)
    parents_positions = Variant.objects.filter(individual__id__in=parents_list).values_list('pos_index', flat=True)

    print('parents_indexes', len(parents_indexes), parents_indexes)
    for individual in parents:
        query['individual_id__in'] = [parents[individual]]
        #parents_variants[individual] = Variant.objects.filter(individual__id=parents[individual], *args, **query).exclude(**exclude)#.values_list('index', flat=True)
        #print 'parents_variants', len(parents_variants[individual])
        individual_variants = Variant.objects.filter(individual__id=parents[individual]).values('chr', 'pos', 'mutation_type', 'gene')
        print('individual variants')
        print(individual, parents[individual])
        print(len(individual_variants)) #all right
        for variant in individual_variants:
            id = '%s-%s' % (variant['chr'], variant['pos'])
            gene = variant['gene']
            #check if gene don't exists in structure first time of gene in structure 
            if gene not in parents_variants[individual]:
                parents_variants[individual][gene] = {}
            parents_variants[individual][gene][id] = {}
            parents_variants[individual][gene][id][variant['mutation_type']] = 0

    children = request.GET.getlist('children')

    children_indexes = Variant.objects.filter(individual__id__in=children).values_list('index', flat=True)
    children_positions = Variant.objects.filter(individual__id__in=children).values_list('pos_index', flat=True)
    #build same dict for children
    print('debug argments', args, query, exclude)
    children_variants = {}
    for child in children:
        print('child', child)
        children_variants[child] = {}
        query['individual_id__in'] = child
        individual_variants = Variant.objects.filter(individual__id=child, *args, **query).exclude(**exclude).values('id', 'chr', 'pos', 'genotype', 'gene')
        print('query', individual_variants.query)
        for variant in individual_variants:
            id = '%s-%s' % (variant['chr'], variant['pos'])
            gene = variant['gene']
            #check if gene don't exists in structure first time of gene in structure 
            if gene not in children_variants[child]:
                children_variants[child][gene] = {}
            children_variants[child][gene][id] = {}
            children_variants[child][gene][id][variant['genotype']] = variant['id']
        print('finished build dict')

    # print 'children_variants', children_variants


    
    inheritance_option = request.GET.get('inheritance_option', '')
    
    # variants = Variant.objects.filter(*args, **query).exclude(**exclude)
    # implement recessive model
    if inheritance_option == 'recessive' or inheritance_option == 'xlinked recessive':
        #this guarantee that same genotype is not at parents
        # args.append(~Q(index__in=parents_indexes))
        #even more efficient add only index that are not present at childs, this will save a lot of query time!
        index_list = []
        # for parent_index in parents_indexes:
        #     if parent_index not in children_indexes:
        #         index_list.append(parent_index)
        # print 'index_list recessive', len(children_indexes)
        # print 'index_list parents', len(parents_indexes)
        # args.append(Q(index__in=index_list))
        index_list = list(set(children_indexes)-set(parents_indexes))
        # print 'index_list', len(index_list)
        args.append(Q(index__in=index_list))

        #ex instead of adding 83k indexes from parents will add only the ones not present at child
    elif inheritance_option == 'recessive denovo':
        #this should guarantee that both parents are genotyped and do not have the variants
        args.append(~Q(pos_index__in=parents_positions))
        #should be
        #args.append(Q(pos_index__in=parents_positions))
        #args.append(~Q(index__in=parents_indexes))
    elif inheritance_option == 'dominant' or inheritance_option == 'xlinked dominant':
        #this is very inefficient
        # args.append(~Q(pos_index__in=parents_positions))
        
        positions_list = list(set(children_positions)-set(parents_positions))
        args.append(Q(pos_index__in=positions_list))



    elif inheritance_option == 'compound heterozygous':
        print('doing compound heterozygous')
        #build children dict with genes and variants
        
        #now search for compound heterozygous magic happens in here!
        exclude_gene_list =[]
        exclude_variant_list=[]
        # print 'opps erro no children'
        # print children
        for child in children:
            #for every gene
            for gene in children_variants[child]:
                #check if at least one variant comes unique from father 
                #and one comes unique from mother
                one_comes_only_from_father = False
                one_comes_only_from_mother = False
                #for every variant in gene from child
                #3 possibilities 
                #two denovo
                #one denovo one from father or mother
                #one from father and one from mother
                if gene in parents_variants['father'] and gene in parents_variants['mother']:
                    # if gene == 'KIF14':
                    #     print 'achou KIF14'
                    #     print child
                    #     print children_variants[child][gene]
                    #     print parents_variants['father'][gene]
                    #     print parents_variants['mother'][gene]
                    #for every variant in child gene
                    for id in children_variants[child][gene]:

                        if (id in parents_variants['father'][gene]) and (id not in parents_variants['mother'][gene]):
                            if 'HOM' not in parents_variants['father'][gene][id]:
                                one_comes_only_from_father = True
                        if (id in parents_variants['mother'][gene]) and (id not in parents_variants['father'][gene]):
                            if 'HOM' not in parents_variants['mother'][gene][id]:
                                one_comes_only_from_mother = True


                        if id in parents_variants['mother'][gene]:
                            if 'HOM' in parents_variants['mother'][gene][id]:
                                exclude_variant_list.append(list(children_variants[child][gene][id].values())[0])
                        if id in parents_variants['father'][gene]:
                            if 'HOM' in parents_variants['father'][gene][id]:
                                exclude_variant_list.append(list(children_variants[child][gene][id].values())[0])
                        #if present in both exclude variants
                        # if remove_in_both_parents == 'on':
                        if (id in parents_variants['father'][gene]) and (id in parents_variants['mother'][gene]):
                            # print 'variant id to remove', children_variants[child][gene][id].values()[0]
                            exclude_variant_list.append(list(children_variants[child][gene][id].values())[0])

                        #if id not seeing in both parents remove from results
                        # if remove_not_in_parents== 'on':
                        #     if id not in parents_variants['mother'][gene]:
                        #         if id not in parents_variants['father'][gene]:
                        #             # print 'Exclui variant %s' % (id)
                        #             exclude_variant_list.append(children_variants[child][gene][id].values()[0])
                    if one_comes_only_from_father and one_comes_only_from_mother:
                        pass
                    else:
                        exclude_gene_list.append(gene)
                #option for removing variants from genes not seing in parents
                else:
                #     if remove_not_in_parents == 'on':
                    exclude_gene_list.append(gene)
                
        print(len(exclude_gene_list))
        if 'gene__in' in exclude:
            exclude['gene__in'] = exclude['gene__in']+exclude_gene_list
        else:
            exclude['gene__in'] = exclude_gene_list
        
        args.append(~Q(id__in=exclude_variant_list))
#       


    return parents_variants
        # print 'parents_variants', parents_variants
    # query['individual_id__in'] = query_bkp

def fill_parents_variants(request, variants, children_positions, parents_variants):

    
    # father = request.GET.get('father', '')
    # mother = request.GET.get('mother', '')
    
    # print 'children_positions', children_positions.count()

    # parents_variants = {'father':{}, 'mother':{}}

    # parents = {}
    # if father != '':
    #     parents['father']=father
    # if mother != '':
    #     parents['mother']=mother


    # #get a small postion of variants from father
    # parents_variants = {'father':{}, 'mother':{}}
    # print 'get position from parents'
    # for individual in parents:
        
    #     individual_variants = Variant.objects.filter(individual__id=parents[individual], pos_index__in=children_positions).values('chr', 'pos', 'mutation_type', 'gene')
    #     print 'parent variants'
    #     print individual, parents[individual]
    #     print len(individual_variants) #all right
    #     for variant in individual_variants:
    #         id = '%s-%s' % (variant['chr'], variant['pos'])
    #         gene = variant['gene']
    #         #check if gene don't exists in structure first time of gene in structure 
    #         if gene not in parents_variants[individual]:
    #             parents_variants[individual][gene] = {}
    #         parents_variants[individual][gene][id] = {}
    #         parents_variants[individual][gene][id][variant['mutation_type']] = 0
    # print 'get position from childs'
    for variant in variants:
        id = '%s-%s' % (variant.chr, variant.pos)
        gene = variant.gene
        if gene in  parents_variants['father']:
            if id in parents_variants['father'][gene]:
                variant.father =  list(parents_variants['father'][gene][id].keys())[0]
        if gene in  parents_variants['mother']:
            if id in parents_variants['mother'][gene]:
                variant.mother =  list(parents_variants['mother'][gene][id].keys())[0]    
    # print 'pass'
#This is the most important function
def family_analysis(request):
    query = {}
    exclude = {}
    summary = {}
    args = []

    #this removes page from QUERY_STRING
    query_string = request.META['QUERY_STRING']
    new_query_string = []
    for item in query_string.split('&'):
        if not item.startswith('page'):
             new_query_string.append(item)
    query_string = "&".join(new_query_string)
    
    # print 'query string'
    # print query_string 
    
    filteranalysis = FilterAnalysis.objects.all().prefetch_related('user')
    filterconfigs = FilterConfig.objects.all().prefetch_related('user')
    
    #this is for checking if query is empty (to avoid loading at the first time)
    if query_string != [''] and query_string != '' :
        
        #make it GET for explicit urls
        if request.method == 'GET':

            form = FamilyAnalysisForm(request.user, request.GET)
            # request.GET = request.GET.copy()

            if 'sort_by' in request.GET:
                sort_by = request.GET['sort_by']
                if sort_by == 'desc':
                    sort = 'asc'
                else:
                    sort = 'desc'
            else:
                sort = 'asc'

            if 'order_by' in request.GET:
                if sort_by == 'desc':
                    order_by = '-%s' % request.GET['order_by']
                else:
                    order_by = request.GET['order_by']
            else:
                order_by = 'gene'

            filter_analysis(request, query, args, exclude)

            individuals_list = filter_individuals_variants(request, query, args, exclude)            

            parents_variants = filter_family_analysis(request, query, args, exclude)

            children = request.GET.getlist('children')
            if len(children) > 0:
                for child in children:
                    individuals_list.append(child)

            if len(individuals_list) > 0:
                #only add to query after filtering the indexes 
                query['individual_id__in'] = individuals_list

                filter_variants_per_gene(request, query, args, exclude)
                filter_genes_in_common(request, query, args, exclude)
                filter_positions_in_common(request, query, args, exclude)
            
            

            #last call to the DATABASE Finally!!!!!! 
            print('last call to database')
            variants = Variant.objects.filter(*args, **query).exclude(**exclude).prefetch_related('individual').order_by(order_by)

            #child_positions
            children_positions = Variant.objects.filter(*args, **query).exclude(**exclude).values_list('pos_index', flat=True)
            
            # export_to_csv()
            print('fill parents variants')
            fill_parents_variants(request, variants, children_positions, parents_variants)
            
            export = request.GET.get('export', '')
            if export != '':
                if export == 'csv':
                    response = HttpResponse(content_type='text/csv')
                    response['Content-Disposition'] = 'attachment; filename=export.csv'
                    writer = csv.writer(response)
                    
                elif export == 'txt':
                    response = HttpResponse(content_type='text/plain')
                    response['Content-Disposition'] = 'attachment; filename=export.txt'
                    writer = csv.writer(response, delimiter='\t', quoting=csv.QUOTE_NONE)    
                writer.writerow(['Individual', 'Index', 'Pos_index', 'Chr', 'Pos', 'Variant_id', 'Ref', 'Alt', 'Qual', 'Filter', 'Info', 'Format', 'Genotype_col', 'Genotype', 'Father Genotype', 'Mother Genotype', 'Read_depth', 'Gene', 'Mutation_type', 'Vartype', 'Genomes1k_maf', 'Dbsnp_maf', 'Esp_maf', 'Dbsnp_build', 'Sift', 'Sift_pred', 'Polyphen2', 'Polyphen2_pred', 'Condel', 'Condel_pred', 'DANN', 'CADD', 'Is_at_omim', 'Is_at_hgmd', 'Hgmd_entries', 'Effect', 'Impact', 'Func_class', 'Codon_change', 'Aa_change', 'Gene_name', 'Biotype', 'Gene_coding', 'Transcript_id', 'Exon_rank', 'Allele', 'Gene', 'Feature', 'Feature_type', 'Consequence', 'Cdna_position', 'Cds_position', 'Protein_position', 'Amino_acids', 'Codons', 'Existing_variation', 'Distance', 'Strand', 'Symbol', 'Symbol_source', 'Sift', 'Polyphen', 'Condel']) 
                for variant in variants:
                    # print 'variant', variant.index
                    writer.writerow([variant.individual, variant.index, variant.pos_index, variant.chr, variant.pos, variant.variant_id, variant.ref, variant.alt, variant.qual, variant.filter, json.loads(variant.info), variant.format, variant.genotype_col, variant.genotype, variant.father, variant.mother, variant.read_depth, variant.gene, variant.mutation_type, variant.vartype, variant.genomes1k_maf, variant.dbsnp_maf, variant.esp_maf, variant.dbsnp_build, variant.sift, variant.sift_pred, variant.polyphen2, variant.polyphen2_pred, variant.condel, variant.condel_pred, variant.dann, variant.cadd, variant.is_at_omim, variant.is_at_hgmd, variant.hgmd_entries, variant.snpeff_effect, variant.snpeff_impact, variant.snpeff_func_class, variant.snpeff_codon_change, variant.snpeff_aa_change, variant.snpeff_gene_name, variant.snpeff_biotype, variant.snpeff_gene_coding, variant.snpeff_transcript_id, variant.snpeff_exon_rank, variant.vep_allele, variant.vep_gene, variant.vep_feature, variant.vep_feature_type, variant.vep_consequence, variant.vep_cdna_position, variant.vep_cds_position, variant.vep_protein_position, variant.vep_amino_acids, variant.vep_codons, variant.vep_existing_variation, variant.vep_distance, variant.vep_strand, variant.vep_symbol, variant.vep_symbol_source, variant.vep_sift, variant.vep_polyphen, variant.vep_condel])
                return response
            #summary
            summary['genes'] = variants.values_list('gene', flat=True)
            summary['genes'] = sorted(list(set(summary['genes'])))
            summary['n_genes'] =  len(summary['genes'])
            summary['has_variants'] = True
            summary['n_variants'] = len(variants)

            genes = get_genes(summary)            

            paginator = Paginator(variants, 100) # Show 25 contacts per page
            try:
               page = int(request.GET.get('page', '1'))
            except ValueError:
               page = 1
            try:
               variants = paginator.page(page)
            except PageNotAnInteger:
               # If page is not an integer, deliver first page.
               variants = paginator.page(1)
            except EmptyPage:
               # If page is out of range (e.g. 9999), deliver last page of results.
               variants = paginator.page(paginator.num_pages)
    #first entrance in methods
    else:
        variants = []
        summary = {}
        genes = {}

        summary['has_variants'] = False
        #fake number just not to open accordion on view 
        summary['n_genes'] = 1000
        
        genes['genes'] = []
        genes['genes_hgmd'] = []
        genes['genes_omim'] = []
        genes['genes_cgd'] = []
        form = FamilyAnalysisForm(request.user)

    return render(request,'filter_analysis/family_analysis.html',
        {'variants':variants, 
        'form':form, 
        'summary':summary,
        'query_string':query_string, 
        'filteranalysis':filteranalysis, 
        'filterconfigs':filterconfigs, 
        'genes':genes['genes'], 
        'genes_hgmd':genes['genes_hgmd'],
        'genes_omim':genes['genes_omim'],
        'genes_cgd':genes['genes_cgd']})




def wizard(request):
    form = FilterWizard([FilterWiZardForm1, FilterWiZardForm2, FilterWiZardForm3])
    if request.method == 'GET':
        print('CHECK HERE')
        query_string = request.META['QUERY_STRING']
        if query_string != '':
            print("LIMPANDO")
            new_query_string = []
            query_string = query_string.split('&')
            for item in query_string:
                if not (item.startswith('csrfmiddlewaretoken') or item.startswith('hash') or item.startswith('wizard')):
                    #get here only the ones need to be cleaned Ex. 1-chr
                    item = "-".join(item.split('-', 2)[1:])
                    new_query_string.append(item)

            #create new query
            filterstring = "&".join(new_query_string)
            # return HttpResponseRedirect('/filter_analysis/?%s' % (filterstring))
            return redirect(reverse('filter_analysis')+'?'+filterstring)
                    
                    
                     
                
                
    return form(context=RequestContext(request), request=request)


class FilterAnalysisUpdateView(UpdateView):
    model = FilterAnalysis    
    def get_success_url(self):
        return reverse('filter_analysis')

class FilterAnalysisDeleteView(DeleteView):
    model = FilterAnalysis    
    def get_success_url(self):
        return reverse('filter_analysis')

class FilterConfigUpdateView(UpdateView):
    model = FilterConfig
    template_name = "filter_analysis/filteranalysis_form.html"    
    def get_success_url(self):
        return reverse('filter_analysis')

class FilterConfigDeleteView(DeleteView):
    model = FilterConfig    
    template_name = "filter_analysis/filteranalysis_confirm_delete.html"
    def get_success_url(self):
        return reverse('filter_analysis')


def create(request):
    print('Hello')
    filterstring = request.META['QUERY_STRING']
    print(filterstring)
    if request.method == 'POST':
        form = Filter(request.POST)        
        if form.is_valid():
            #use id for unique names
            filter = FilterAnalysis.objects.create(user=request.user)
            filter.name = request.POST['name']
            filter.filterstring = form.cleaned_data['filterstring']
            filter.save()
            #return HttpResponseRedirect('/filter_analysis/?%s' % (filter.filterstring))
            return redirect(reverse('filter_analysis')+'?'+filter.filterstring)
    else:
        form = Filter(initial={'filterstring': filterstring})
        
    return render(request, 'filter_analysis/createfilter.html', {'form': form})


def family_analysis_create_filter(request):
    print('Hello')
    filterstring = request.META['QUERY_STRING']
    print(filterstring)
    if request.method == 'POST':
        form = FamilyFilter(request.POST)        
        if form.is_valid():
            #use id for unique names
            filter = FamilyFilterAnalysis.objects.create(user=request.user)
            filter.name = request.POST['name']
            filter.filterstring = form.cleaned_data['filterstring']
            filter.save()
            #return HttpResponseRedirect('/filter_analysis/family_analysis/?%s' % (filter.filterstring))
            return redirect(reverse('family_analysis')+'?'+filter.filterstring)
            # return redirect('mendelmd/%s' % (reverse('family_analysis')+'?'+filter.filterstring))
    else:
        form = FamilyFilter(initial={'filterstring': filterstring})
        
    return render(request, 'filter_analysis/createfilter.html', {'form': form})


def createconfig(request):
    print('Hello Config')
    query_string = request.META['QUERY_STRING']
    new_query_string = []
    for item in query_string.split('&'):
        if not item.startswith('individuals'):
             new_query_string.append(item)
    query_string = "&".join(new_query_string)
    filterstring = query_string

    if request.method == 'POST':
        form = Filter(request.POST)        
        if form.is_valid():
            #use id for unique names
            filterconfig = FilterConfig.objects.create(user=request.user)
            filterconfig.name = request.POST['name']
            filterconfig.filterstring = form.cleaned_data['filterstring']
            filterconfig.save()
            #return HttpResponseRedirect('/filter_analysis/?%s' % (filterconfig.filterstring))
            return redirect(reverse('filter_analysis')+'?'+filterconfig.filterstring)
    else:
        form = Filter(initial={'filterstring': filterstring})
        
    return render(request, 'filter_analysis/createfilter.html', {'form': form})





from django.db import models

# Create your models here.
class Pathway(models.Model):
    kegg = models.CharField(max_length=255)
    name = models.TextField(null=True, blank=True)
    genes = models.TextField(null=True, blank=True)

#GenePathway


"""
This file demonstrates writing tests using the unittest module. These will pass
when you run "manage.py test".

Replace this with more appropriate tests for your application.
"""

from django.test import TestCase

from individuals.models import Individual

class SimpleTest(TestCase):
    def test_basic_addition(self):
        """
        Tests that 1 + 1 always equals 2.
        """
        self.assertEqual(1 + 1, 2)
    def test_individuals_filter(self):
        """
        Tests get individuals variants
        """
        # individuals_list = ['Exome_4_ELS.var.annotated', 'Exome_3_EDS.var.annotated'],
        # print 'hello world'
        # individuals = Individual.objects.all()
        # print individuals
        # for individual in individuals:
        #     print individual.name
        # print individual
        self.assertEqual(1 + 1, 2)



from django import template

register = template.Library()


@register.filter
def paginator(context, adjacent_pages=2):
    """
    To be used in conjunction with the object_list generic view.

    Adds pagination context variables for use in displaying first, adjacent and
    last page links in addition to those created by the object_list generic
    view.

    """
    page_numbers = [n for n in \
                    range(context['page'] - adjacent_pages, context['page'] + adjacent_pages + 1) \
                    if n > 0 and n <= context['pages']]
    return {
        'hits': context['hits'],
        'results_per_page': context['results_per_page'],
        'page': context['page'],
        'pages': context['pages'],
        'page_numbers': page_numbers,
        'next': context['next'],
        'previous': context['previous'],
        'has_next': context['has_next'],
        'has_previous': context['has_previous'],
        'show_first': 1 not in page_numbers,
        'show_last': context['pages'] not in page_numbers,
    }

#register.inclusion_tag('paginator.html', takes_context=True)(paginator)


from django.db import models
from individuals.models import Individual

class Variant(models.Model):

    individual = models.ForeignKey(Individual, on_delete=models.CASCADE)

    index = models.TextField(db_index=True)#ex. 1-2387623-G-T
    pos_index = models.TextField(db_index=True)#ex. 1-326754756

    #First save all 9 VCF columns
    chr = models.TextField(verbose_name="Chr", db_index=True)
    pos = models.IntegerField(db_index=True)
    variant_id = models.TextField(verbose_name="ID", db_index=True)
    ref = models.TextField(null=True, blank=True, db_index=True)
    alt = models.TextField(null=True, blank=True, db_index=True)
    qual = models.FloatField(db_index=True)
    filter = models.TextField(db_index=True)
    info = models.TextField(null=True, blank=True)
    format = models.TextField(null=True, blank=True, db_index=True)

    genotype_col = models.TextField(null=True, blank=True, db_index=True)
    genotype = models.TextField(db_index=True)

    #metrics from genotype_info DP field
    read_depth = models.IntegerField()

    gene = models.TextField(null=True, blank=True, db_index=True)
    mutation_type = models.TextField(null=True, db_index=True)
    vartype = models.TextField(null=True, db_index=True)

    #Annotation From 1000genomes
    genomes1k_maf = models.FloatField(null=True, blank=True, verbose_name="1000 Genomes Frequency", db_index=True)
    dbsnp_maf = models.FloatField(null=True, blank=True, verbose_name="dbSNP Frequency", db_index=True)
    esp_maf = models.FloatField(null=True, blank=True, verbose_name="ESP6500 Frequency", db_index=True)
    

    #dbsnp
    # dbsnp_pm = models.TextField(null=True, blank=True)
    # dbsnp_clnsig = models.TextField(null=True, blank=True)
    dbsnp_build = models.IntegerField(null=True, db_index=True)

    #VEP
    sift = models.FloatField(null=True, blank=True, db_index=True)
    sift_pred = models.TextField(null=True, blank=True, db_index=True)

    polyphen2 = models.FloatField(null=True, blank=True, db_index=True)
    polyphen2_pred = models.TextField(null=True, blank=True, db_index=True)

    condel = models.FloatField(null=True, blank=True, db_index=True)
    condel_pred = models.TextField(null=True, blank=True, db_index=True)

    dann = models.FloatField(null=True, blank=True, db_index=True)
    cadd = models.FloatField(null=True, blank=True, db_index=True)

    #hi_index
    hi_index_str = models.TextField(null=True, blank=True, db_index=True)
    hi_index = models.FloatField(null=True, blank=True, db_index=True)
    hi_index_perc = models.FloatField(null=True, blank=True, db_index=True)

    #OMIM
    is_at_omim = models.BooleanField(default=False, db_index=True)

    #HGMD
    is_at_hgmd = models.BooleanField(default=False, db_index=True)
    hgmd_entries = models.TextField(null=True, blank=True, db_index=True)

    #snpeff annotation
    snpeff_effect = models.TextField(null=True, blank=True, db_index=True)
    snpeff_impact = models.TextField(null=True, blank=True, db_index=True)
    snpeff_func_class = models.TextField(null=True, blank=True, db_index=True)
    snpeff_codon_change = models.TextField(null=True, blank=True, db_index=True)
    snpeff_aa_change = models.TextField(null=True, blank=True, db_index=True)
    # snpeff_aa_len = models.TextField(null=True, blank=True)
    snpeff_gene_name = models.TextField(null=True, blank=True, db_index=True)
    snpeff_biotype = models.TextField(null=True, blank=True, db_index=True)
    snpeff_gene_coding = models.TextField(null=True, blank=True, db_index=True)
    snpeff_transcript_id = models.TextField(null=True, blank=True, db_index=True)
    snpeff_exon_rank = models.TextField(null=True, blank=True, db_index=True)
    # snpeff_genotype_number = models.TextField(null=True, blank=True)

    #vep annotation
    vep_allele = models.TextField(null=True, blank=True, db_index=True)
    vep_gene = models.TextField(null=True, blank=True, db_index=True)
    vep_feature = models.TextField(null=True, blank=True, db_index=True)
    vep_feature_type = models.TextField(null=True, blank=True, db_index=True)
    vep_consequence = models.TextField(null=True, blank=True, db_index=True)
    vep_cdna_position = models.TextField(null=True, blank=True, db_index=True)
    vep_cds_position = models.TextField(null=True, blank=True, db_index=True)
    vep_protein_position = models.TextField(null=True, blank=True, db_index=True)
    vep_amino_acids = models.TextField(null=True, blank=True, db_index=True)
    vep_codons = models.TextField(null=True, blank=True, db_index=True)
    vep_existing_variation = models.TextField(null=True, blank=True, db_index=True)
    vep_distance = models.TextField(null=True, blank=True, db_index=True)
    vep_strand = models.TextField(null=True, blank=True, db_index=True)
    vep_symbol = models.TextField(null=True, blank=True, db_index=True)
    vep_symbol_source = models.TextField(null=True, blank=True, db_index=True)
    vep_sift = models.TextField(null=True, blank=True, db_index=True)
    vep_polyphen = models.TextField(null=True, blank=True, db_index=True)
    vep_condel = models.TextField(null=True, blank=True, db_index=True)

    #new annotations
    ensembl_clin_HGMD = models.BooleanField(default=False, db_index=True)
    ensembl_clin_HGMD = models.BooleanField(default=False, db_index=True)
    clinvar_CLNSRC = models.TextField(null=True, blank=True, db_index=True)
    # ensembl_phen.CLIN_pathogenic
    #ensembl_phen.CLIN_likely_pathogenic
    # ensembl_clin.CLIN_pathogenic

    #DBNFSP
    SIFT_score = models.TextField(null=True, blank=True, db_index=True)
    SIFT_converted_rankscore = models.TextField(null=True, blank=True, db_index=True)
    # SIFT_pred = models.TextField(null=True, blank=True)
    Uniprot_acc_Polyphen2 = models.TextField(null=True, blank=True, db_index=True)
    Uniprot_id_Polyphen2 = models.TextField(null=True, blank=True, db_index=True)
    Uniprot_aapos_Polyphen2 = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HDIV_score = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HDIV_rankscore = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HDIV_pred = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HVAR_score = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HVAR_rankscore = models.TextField(null=True, blank=True, db_index=True)
    Polyphen2_HVAR_pred = models.TextField(null=True, blank=True, db_index=True)
    LRT_score = models.TextField(null=True, blank=True, db_index=True)
    LRT_converted_rankscore = models.TextField(null=True, blank=True, db_index=True)
    LRT_pred = models.TextField(null=True, blank=True, db_index=True)
    LRT_Omega = models.TextField(null=True, blank=True, db_index=True)
    MutationTaster_score = models.TextField(null=True, blank=True, db_index=True)
    MutationTaster_converted_rankscore = models.TextField(null=True, blank=True, db_index=True)
    MutationTaster_pred = models.TextField(null=True, blank=True, db_index=True)
    MutationTaster_model = models.TextField(null=True, blank=True, db_index=True)
    MutationTaster_AAE = models.TextField(null=True, blank=True, db_index=True)
    MutationAssessor_UniprotID = models.TextField(null=True, blank=True, db_index=True)
    MutationAssessor_variant = models.TextField(null=True, blank=True, db_index=True)
    MutationAssessor_score = models.TextField(null=True, blank=True, db_index=True)
    MutationAssessor_rankscore = models.TextField(null=True, blank=True, db_index=True)
    MutationAssessor_pred = models.TextField(null=True, blank=True, db_index=True)
    FATHMM_score = models.TextField(null=True, blank=True, db_index=True)
    FATHMM_converted_rankscore = models.TextField(null=True, blank=True, db_index=True)
    FATHMM_pred = models.TextField(null=True, blank=True, db_index=True)
    PROVEAN_score = models.TextField(null=True, blank=True, db_index=True)
    PROVEAN_converted_rankscore = models.TextField(null=True, blank=True, db_index=True)
    PROVEAN_pred = models.TextField(null=True, blank=True, db_index=True)
    Transcript_id_VEST3 = models.TextField(null=True, blank=True, db_index=True)
    Transcript_var_VEST3 = models.TextField(null=True, blank=True, db_index=True)
    VEST3_score = models.TextField(null=True, blank=True, db_index=True)
    VEST3_rankscore = models.TextField(null=True, blank=True, db_index=True)
    MetaSVM_score = models.TextField(null=True, blank=True, db_index=True)
    MetaSVM_rankscore = models.TextField(null=True, blank=True, db_index=True)
    MetaSVM_pred = models.TextField(null=True, blank=True, db_index=True)
    MetaLR_score = models.TextField(null=True, blank=True, db_index=True)
    MetaLR_rankscore = models.TextField(null=True, blank=True, db_index=True)
    MetaLR_pred = models.TextField(null=True, blank=True, db_index=True)
    Reliability_index = models.TextField(null=True, blank=True, db_index=True)
    CADD_raw = models.TextField(null=True, blank=True, db_index=True)
    CADD_raw_rankscore = models.TextField(null=True, blank=True, db_index=True)
    CADD_phred = models.TextField(null=True, blank=True, db_index=True)
    DANN_score = models.TextField(null=True, blank=True, db_index=True)
    DANN_rankscore = models.TextField(null=True, blank=True, db_index=True)
    fathmm_MKL_coding_score = models.TextField(null=True, blank=True, db_index=True)
    fathmm_MKL_coding_rankscore = models.TextField(null=True, blank=True, db_index=True)
    fathmm_MKL_coding_pred = models.TextField(null=True, blank=True, db_index=True)
    fathmm_MKL_coding_group = models.TextField(null=True, blank=True, db_index=True)
    Eigen_raw = models.TextField(null=True, blank=True, db_index=True)
    Eigen_phred = models.TextField(null=True, blank=True, db_index=True)
    Eigen_raw_rankscore = models.TextField(null=True, blank=True, db_index=True)
    Eigen_PC_raw = models.TextField(null=True, blank=True, db_index=True)
    Eigen_PC_raw_rankscore = models.TextField(null=True, blank=True, db_index=True)
    GenoCanyon_score = models.TextField(null=True, blank=True, db_index=True)
    GenoCanyon_score_rankscore = models.TextField(null=True, blank=True, db_index=True)
    integrated_fitCons_score = models.TextField(null=True, blank=True, db_index=True)
    integrated_fitCons_rankscore = models.TextField(null=True, blank=True, db_index=True)
    integrated_confidence_value = models.TextField(null=True, blank=True, db_index=True)
    GM12878_fitCons_score = models.TextField(null=True, blank=True, db_index=True)
    GM12878_fitCons_rankscore = models.TextField(null=True, blank=True, db_index=True)
    GM12878_confidence_value = models.TextField(null=True, blank=True, db_index=True)
    H1_hESC_fitCons_score = models.TextField(null=True, blank=True, db_index=True)
    H1_hESC_fitCons_rankscore = models.TextField(null=True, blank=True, db_index=True)
    H1_hESC_confidence_value = models.TextField(null=True, blank=True, db_index=True)
    HUVEC_fitCons_score = models.TextField(null=True, blank=True, db_index=True)
    HUVEC_fitCons_rankscore = models.TextField(null=True, blank=True, db_index=True)
    HUVEC_confidence_value = models.TextField(null=True, blank=True, db_index=True)
    GERP_NR = models.TextField(null=True, blank=True, db_index=True)
    GERP_RS = models.TextField(null=True, blank=True, db_index=True)
    GERP_RS_rankscore = models.TextField(null=True, blank=True, db_index=True)
    phyloP100way_vertebrate = models.TextField(null=True, blank=True, db_index=True)
    phyloP100way_vertebrate_rankscore = models.TextField(null=True, blank=True, db_index=True)
    phyloP20way_mammalian = models.TextField(null=True, blank=True, db_index=True)
    phyloP20way_mammalian_rankscore = models.TextField(null=True, blank=True, db_index=True)
    phastCons100way_vertebrate = models.TextField(null=True, blank=True, db_index=True)
    phastCons100way_vertebrate_rankscore = models.TextField(null=True, blank=True, db_index=True)
    phastCons20way_mammalian = models.TextField(null=True, blank=True, db_index=True)
    phastCons20way_mammalian_rankscore = models.TextField(null=True, blank=True, db_index=True)
    SiPhy_29way_pi = models.TextField(null=True, blank=True, db_index=True)
    SiPhy_29way_logOdds = models.TextField(null=True, blank=True, db_index=True)
    SiPhy_29way_logOdds_rankscore = models.TextField(null=True, blank=True, db_index=True)
    clinvar_rs = models.TextField(null=True, blank=True, db_index=True)
    clinvar_clnsig = models.TextField(null=True, blank=True, db_index=True)
    clinvar_trait = models.TextField(null=True, blank=True, db_index=True)
    clinvar_golden_stars = models.TextField(null=True, blank=True, db_index=True)
    mcap_score = models.FloatField(null=True, blank=True, db_index=True)
    mcap_rankscore = models.FloatField(null=True, blank=True, db_index=True)
    mcap_pred = models.TextField(null=True, blank=True, db_index=True)
    revel_score = models.TextField(null=True, blank=True, db_index=True)


    def get_fields(self):
    	return [(field.name, field.verbose_name.title().replace('_', ' ')) for field in Variant._meta.fields]




from django.shortcuts import render, redirect, get_object_or_404

from .models import Project
from files.models import File

from tasks.models import Task

from .forms import ProjectForm, ImportForm
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.views.generic.detail import DetailView
from django.contrib.auth.mixins import LoginRequiredMixin

from django.contrib.auth.decorators import login_required

from django.urls import reverse_lazy

from tasks.tasks import run_qc
from files.tasks import check_file

from .tasks import import_project_files_task

from settings.models import S3Credential

import boto3
import os

from .models import ProjectFile, ProjectSample


from django.utils.decorators import method_decorator

from django.core import serializers

@login_required
def index(request):

    if request.user.is_staff:
        projects = Project.objects.all()
    else:
        projects = Project.objects.filter(user=request.user)

    context = {'projects':projects}
    return render(request, 'projects/index.html', context)

@login_required
def create(request):
    """
    Create Project
    """
    form = ProjectForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            project = form.save(commit=False)
            project.user = request.user
            project.save()

            return redirect('projects-view', project_id=project.id)

    context = {'form': form}
    return render(request, 'projects/create.html', context)

@method_decorator(login_required, name='dispatch')
class ProjectUpdate(UpdateView):
    model = Project
    fields = ['name', 'paths']

    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

@login_required
def view(request, project_id):

    if request.user.is_staff:
        project = get_object_or_404(Project, pk=project_id)
    else:
        project = get_object_or_404(Project, pk=project_id, user=request.user)

    print(dir(project))

    n_files = project.files.count()
    n_samples = project.files.count()
    # total_file_size = 0
    # total_file_size = sum(project.files.values_list('size', flat=True))
    
    context = {
        'project': project,
        'n_files':n_files,
        'n_samples':n_samples,
        # 'total_file_size':total_file_size,
    }

    return render(request, 'projects/view.html', context)

@login_required
def project_files(request, project_id):

    if request.user.is_staff:
        project = get_object_or_404(Project, pk=project_id)
    else:
        project = get_object_or_404(Project, pk=project_id, user=request.user)

    n_files = project.projectfile_set.count()

    context = {
        'project': project,
        'n_files':n_files
    }
    
    return render(request, 'projects/project_files.html', context)


@login_required
def import_files(request, project_id):
    if request.user.is_staff:
        project = get_object_or_404(Project, pk=project_id)
    else:
        project = get_object_or_404(Project, pk=project_id, user=request.user)

    form = ImportForm(request.POST or None)
    if request.method == 'POST':
        if form.is_valid():
            # paths = form.cleaned_data['paths']

            for file in form.cleaned_data['file_list'].splitlines():
                
                try:
                    obj = File.objects.get(location=file, user=request.user)
                except File.DoesNotExist:
                    obj = File(location=file, user=request.user)
                    obj.save()
                    project.files.add(obj)

            # for path in form.cleaned_data['paths'].splitlines():
                
            #     clean_path = path.strip().replace('s3://', '')
            #     split_path = clean_path.split('/', 1)
            #     bucket_name = split_path[0]
            #     prefix = split_path[1]

            #     s3credentials = S3Credential.objects.all()
            #     for s3credential in s3credentials:
            #         if clean_path.startswith(s3credential.buckets):
            #             #get all files from path
            #             session = boto3.Session(
            #                 aws_access_key_id=s3credential.access_key,
            #                 aws_secret_access_key=s3credential.secret_key
            #             )
            #             s3 = session.resource('s3')
            #             bucket = s3.Bucket(bucket_name)
            #             print(bucket)
            #             for key in bucket.objects.filter(Prefix=prefix):
                            

            #                 if not key.key.endswith('/'):
            #                     file_name, file_extension = os.path.splitext(key.key)
            #                     if file_extension == '.gz':
            #                         file_name, file_extension = os.path.splitext(file_name)
                                
            #                     basename = os.path.basename(file_name)
            #                     location = 's3://'+bucket_name+'/'+key.key

            #                     file_obj = ProjectFile(
            #                         project = project,
            #                         name=basename,
            #                         size=str(key.size),
            #                         last_modified=str(key.last_modified),
            #                         file_type=file_extension.replace('.', ''),
            #                         location=location,
            #                         user=request.user
            #                     )

            #                     file_obj.save()

            for sample_info in form.data['samples'].splitlines():
                sample_info = sample_info.split('\t')

                sample = ProjectSample(user=request.user, project=project)
                
                sample.name = sample_info[0]
                sample.alias = sample_info[1]
                sample.status = sample_info[2]
                sample.location = sample_info[3]
                sample.prefix = sample_info[4]
                sample.save()

            return redirect('projects-view', project.id)
    context = {'form': form, 'project': project}
    return render(request, 'projects/import_files.html', context)

@method_decorator(login_required, name='dispatch')
class ProjectDelete(DeleteView):
    model = Project
    success_url = reverse_lazy('projects-index')

    def get_queryset(self):
        if not self.request.user.is_staff:
            return self.model.objects.filter(user=self.request.user)
        else:
            return self.model.objects

@login_required
def import_project_files(request, project_id):
    project = Project.objects.get(pk=project_id)
    if project.user == request.user:
        import_project_files_task.delay(project_id)
    return redirect('projects-view', project_id=project_id)


@login_required
def bulk_action(request, project_id):

    if request.user.is_staff:
        project = get_object_or_404(Project, pk=project_id)
    else:
        project = get_object_or_404(Project, pk=project_id, user=request.user)

    if request.method == 'POST':
        
        action = request.POST['action']

        # print('POST', request.POST)

        model = request.POST['model']


        if model == 'files':
    
            files = request.POST.getlist('files')


            if action == 'analysis':

                request.session['files'] = files
                request.session['project_id'] = project_id
                
                return redirect('analysis-create')


            for file_id in files:
                file = File.objects.get(pk=file_id)
                if action == "delete":
                    file.delete()
                if action == "qc":
                    if file.file_type in ['fq', 'fastq', 'bam', 'vcf']:
                        task = Task()
                        task.name = 'QC ProjectFile %s' % (file.id)
                        task.type = 'qc'
                        task.action = 'qc'
                        task.status = 'new'
                        task.manifest = {
                            'input':file.location,
                            'project':project.id,
                            'file':file.id,
                            'type':'qc',
                        }
                        task.save()
                        run_qc.delay(task.id)
                if action == "check":
                   task_manifest = {}
                   task_manifest['file'] = file.id
                   task_manifest['action'] = action
                   task = Task(user=request.user)
                   task.manifest = task_manifest
                   task.status = 'new'
                   task.action = action
                   task.user = request.user
                   task.save()
                   check_file.delay(task.id)
                   file.status = 'scheduled'
                   file.save()







        if model == 'samples':
            samples = request.POST.getlist('samples')
            for sample_id in samples:
                sample = ProjectSample.objects.get(pk=sample_id)
                if action == "delete":
                    sample.delete()

            
    return redirect('projects-view', project_id=project_id)



from django.db import models

from diseases.models import Disease
from django.contrib.auth.models import User


# Create your models here.
class Gene(models.Model):

    hgnc_id = models.TextField(blank=True)
    symbol = models.TextField(blank=True)
    name = models.TextField(blank=True)
    locus_group = models.TextField(blank=True)
    locus_type = models.TextField(blank=True)
    status = models.TextField(blank=True)
    location = models.TextField(blank=True)
    location_sortable = models.TextField(blank=True)
    alias_symbol = models.TextField(blank=True)
    alias_name = models.TextField(blank=True)
    prev_symbol = models.TextField(blank=True)
    prev_name = models.TextField(blank=True)
    gene_family = models.TextField(blank=True)
    gene_family_id = models.TextField(blank=True)
    date_approved_reserved = models.TextField(blank=True)
    date_symbol_changed = models.TextField(blank=True)
    date_name_changed = models.TextField(blank=True)
    date_modified = models.TextField(blank=True)
    entrez_id = models.TextField(blank=True)
    ensembl_gene_id = models.TextField(blank=True)
    vega_id = models.TextField(blank=True)
    ucsc_id = models.TextField(blank=True)
    ena = models.TextField(blank=True)
    refseq_accession = models.TextField(blank=True)
    ccds_id = models.TextField(blank=True)
    uniprot_ids = models.TextField(blank=True)
    pubmed_id = models.TextField(blank=True)
    mgd_id = models.TextField(blank=True)
    rgd_id = models.TextField(blank=True)
    lsdb = models.TextField(blank=True)
    cosmic = models.TextField(blank=True)
    omim_id = models.TextField(blank=True)
    mirbase = models.TextField(blank=True)
    homeodb = models.TextField(blank=True)
    snornabase = models.TextField(blank=True)
    bioparadigms_slc = models.TextField(blank=True)
    orphanet = models.TextField(blank=True)
    pseudogene_org = models.TextField(blank=True)
    horde_id = models.TextField(blank=True)
    merops = models.TextField(blank=True)
    imgt = models.TextField(blank=True)
    iuphar = models.TextField(blank=True)
    kznf_gene_catalog = models.TextField(blank=True)
    mamit_trnadb = models.TextField(blank=True)
    cd = models.TextField(blank=True)
    lncrnadb = models.TextField(blank=True)
    enzyme_id = models.TextField(blank=True)
    intermediate_filament_db = models.TextField(blank=True)

    diseases = models.ManyToManyField(Disease, related_name='+')
    
    def get_fields(self):
        return [(field.name, field.value_to_string(self)) for field in Gene._meta.fields]
    
    
class GeneCategory(models.Model):
    domain = models.CharField(max_length=255, blank=True)
    name = models.TextField(blank=True)
    go = models.CharField(max_length=255)
    definition = models.TextField(blank=True)
    genes = models.ManyToManyField(Gene)
    
    
class Membership(models.Model):
    gene = models.ForeignKey(Gene, on_delete=models.CASCADE)
    group = models.ForeignKey(GeneCategory, on_delete=models.CASCADE)
    
    
class GeneGroup(models.Model):
    name = models.CharField(max_length=255)
    genes = models.TextField(blank=True)
    
    def __str__(self):
        return self.name
        
        
class GoTerm(models.Model):
    goid = models.CharField(max_length=255)
    name = models.CharField(max_length=255)
    namespace = models.CharField(max_length=255)
    parents = models.ManyToManyField("self", blank=True, symmetrical=False, related_name='parents_go')
    children = models.ManyToManyField("self", blank=True, symmetrical=False, related_name='children_go')
    level = models.CharField(max_length=255) 
    is_obsolete = models.BooleanField(default=True)
    alt_ids = models.CharField(max_length=255)
    
    def __str__(self):
        return self.name



class Manifestation(models.Model):
    name = models.TextField(blank=True)

    def __str__(self):
        return self.name


class Intervention(models.Model):
    name = models.TextField(blank=True)

    def __str__(self):
        return self.name    


class CGDCondition(models.Model):
    name = models.TextField(blank=True)

    def __str__(self):
        return self.name


class CGDEntry(models.Model):

    GENE = models.TextField(blank=True)
    HGNC_ID = models.TextField(blank=True)

    ENTREZ_GENE_ID = models.TextField(blank=True)
    CONDITIONS = models.ManyToManyField(CGDCondition, blank=True)
    INHERITANCE = models.TextField(blank=True)
    AGE_GROUP = models.TextField(blank=True)
    ALLELIC_CONDITIONS = models.TextField(blank=True)

    COMMENTS = models.TextField(blank=True)
    INTERVENTION_RATIONALE = models.TextField(blank=True)
    REFERENCES = models.TextField(blank=True)


    MANIFESTATION_CATEGORIES = models.ManyToManyField(Manifestation, blank=True)

    INTERVENTION_CATEGORIES = models.ManyToManyField(Intervention, blank=True)

    def __str__(self):
        return self.GENE

class GeneList(models.Model):
    name = models.CharField(max_length=255, blank=True)
    genes = models.TextField(blank=True)
    user = models.ForeignKey(User, editable=False, on_delete=models.CASCADE)
    def __str__(self):
            return self.name



from django.conf.urls import *
from django.views.generic import DetailView, ListView, CreateView, UpdateView, DeleteView

from filter_analysis.models import *

from filter_analysis.views import FilterAnalysisUpdateView, FilterAnalysisDeleteView, FilterConfigUpdateView, FilterConfigDeleteView

from filter_analysis import views

urlpatterns = [

    url(r'^$', views.index, name='filter_analysis'),
    url(r'^table/$', views.filter_analysis_table, name='filter_analysis_table'),
    url(r'^family_analysis/$', views.family_analysis, name='family_analysis'),
    url(r'^family_analysis/create/$', views.family_analysis_create_filter, name='family_analysis_create_filter'),  
    url(r'^wizard/$', views.wizard, name='filter_wizard'),
    url(r'^oneclick/$', views.oneclick, name='oneclick'),
    url(r'^test/$', views.test, name='test'),    
    url(r'^create/$', views.create, name='create_filter'),
    url(r'^createconfig/$', views.createconfig, name='create_config'),
    url(r'^updateanalysis/(?P<pk>\d+)', FilterAnalysisUpdateView.as_view(model=FilterAnalysis), name="analysis_update",),
    url(r'^deleteanalysis/(?P<pk>\d+)', FilterAnalysisDeleteView.as_view(), name="analysis_delete",),
    url(r'^updateconfig/(?P<pk>\d+)', FilterConfigUpdateView.as_view(model=FilterConfig), name="config_update",),
    url(r'^deleteconfig/(?P<pk>\d+)', FilterConfigDeleteView.as_view(), name="config_delete",),
]



from django.shortcuts import render
from django.shortcuts import render, redirect, get_object_or_404


from .models import Analysis
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.views.generic.detail import DetailView
from django.urls import reverse_lazy

from django.shortcuts import render
from formtools.wizard.views import SessionWizardView

from tasks.tasks import run_qc
from .tasks import create_analysis_tasks

from projects.models import Project
from files.models import File
from .forms import CreateAnalysis
from tasks.models import Task
from samples.models import SampleGroup
from django.utils.html import strip_tags

# Create your views here.
def index(request):

    query  = ''
    args = []

    if request.method == 'POST':
        
        analyses = request.POST.getlist('analyses')
        action = request.POST['action']
        query = request.POST['query']
        # print('query', query)
        for analysis_id in analyses:

            # file = File.objects.get(pk=file_id)
            if request.user.is_staff:
                analysis = get_object_or_404(Analysis, pk=analysis_id)
            else:
                analysis = get_object_or_404(Analysis, pk=analysis_id, user=request.user)

            if action == "delete":
                analysis.delete()

    print('Hello World')
    analyses = Analysis.objects.all()

    context = {'analyses': analyses}

    return render(request, 'analyses/index.html', context)

def create(request):
    params = {}
    
    if 'files' in request.session:
        file_list = request.session['files']
        files = File.objects.filter(pk__in=file_list)
    else:
        file_list = None
        files = None

    files = File.objects.all()

    if 'project_id' in request.session:

        project_id = request.session['project_id']
        project = Project.objects.get(pk=project_id)
    else:
        project = None

    if request.method == 'POST':
        
        form = CreateAnalysis(request.POST)
        print(request.POST)
        
        if form.is_valid():

            analysis = Analysis(user=request.user)
            
            analysis.name = form.cleaned_data['name']
            

            # analysis.files = form.cleaned_data['files']

            # analysis.analysis_types = form.cleaned_data['analysis_types']
            params['providers'] = form.cleaned_data['providers']
            params['analysis_types'] = form.cleaned_data['analysis_types']
            params['files'] = strip_tags(form.cleaned_data['files'].replace('<br>', '\n')).strip().split('\n')
            
            

            # file_list#
            analysis.status = 'new'
            analysis.project = project
            analysis.params = params

            analysis.save()

            create_analysis_tasks.delay(analysis.id)

            # task_manifest = params
            # task = Task(user=request.user)
            # task.manifest = task_manifest
            # task.status = 'new'
            # task.action = 'analysis'
            # task.save()

            return redirect('analysis-detail', analysis.id)
            
    else:
        form = CreateAnalysis()
    
    # print(file)
    # print(project.name)

    # if request.POST:
        # params = request.POST['params']
        # print(params)

    context = {
        'project': project,
        # 'files': files,
        'form':form,
        }

    return render(request, 'analyses/create.html', context)


class AnalysisDelete(DeleteView):
    model = Analysis
    success_url = reverse_lazy('analyses-index')

# class AnalysisCreate(CreateView):
#     model = Analysis
#     fields = ['name']

class AnalysisUpdate(UpdateView):
    model = Analysis
    fields = ['name', 'params']

class AnalysisDetailView(DetailView):
    model = Analysis
    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super().get_context_data(**kwargs)
        # Add in a QuerySet of all the books
        # print('params', self.object.params)
        files = []
        if 'sample_groups' in self.object.params:
            # for group in self.object.params['sample_groups']:
                # files = File.objects.filter()
                sample_groups = self.object.params['sample_groups']
                samples = SampleGroup.objects.filter(pk__in=sample_groups).values_list('members', flat=True)
                # print('samples', samples)
                context['files'] = File.objects.filter(sample__in=samples)
                # print(context['files'])
        if 'files' in self.object.params:
            files = self.object.params['files']
            # context['files'] = File.objects.filter(pk__in=files)
        context['tasks'] = Task.objects.filter(analysis=self.object)

        context['output_files'] = File.objects.filter(task__in=context['tasks'])
        print(len(context['output_files']))
        # for file in context['output_files']:
        #     print(dir(file))
            
        return context

def human_size(bytes, units=[' bytes','KB','MB','GB','TB', 'PB', 'EB']):
    """ Returns a human readable string reprentation of bytes"""
    return str(bytes) + units[0] if bytes < 1024 else human_size(bytes>>10, units[1:])

def run_analysis(request, analysis_id):
    
    print('run analysis')
    create_analysis_tasks.delay(analysis_id)

    return redirect('analysis-detail', analysis_id)
    


class ContactWizard(SessionWizardView):
    def done(self, form_list, **kwargs):

        return render(self.request, 'done.html', {
            'form_data': [form.cleaned_data for form in form_list],
        })



# -*- coding: utf-8 -*-
from django.http import HttpResponseRedirect
from django.http import HttpResponse
from django.shortcuts import redirect, render
from django.template import RequestContext
from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404
from django.contrib import messages
from django.db.models import Count, Sum, Avg, Min, Max
from django.views.generic import ListView, DetailView
from django.contrib.admin.views.decorators import staff_member_required

from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
#from django.views.generic.detail import DetailView

# Imaginary function to handle an uploaded file.
#from somewhere import handle_uploaded_file
from diseases.models import *
from individuals.models import *
from variants.models import *
import codecs
from django.db.models import Q
#from mysql_bulk_insert import bulk_insert
import re
from .forms import *
from collections import Counter
#import settings
# from goatools import obo_parser
from genes.models import *
from django.db import transaction
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django.utils.decorators import method_decorator
from django.http import Http404
from django.urls import reverse
from django.conf import settings
from django.shortcuts import render


def genesetcreate(request):
    form = GenesetForm(request.POST or None)
    
    if request.method == 'POST' and form.is_valid():
            genelist = GeneList()
            genelist.name = form.cleaned_data['name'] 


            genelist_list = form.cleaned_data['genes']

            genelist_list = genelist_list.split('\r\n')
            # print 'gene_list'
            if genelist_list[0] != '':
                safe_gene_list = []
                for row in genelist_list:
                    row = row.split(',')
                    for item in row:
                        gene = item.strip()
                        if (gene != ' ' and gene != ''):
                            if gene not in safe_gene_list:
                                safe_gene_list.append(item.strip())

            genelist.genes = ",".join(safe_gene_list)
            #form.cleaned_data['genes']



            genelist.user=request.user
            genelist.save()
            # form.save()

            
            
            return redirect(reverse('genes_list'))

    # context = RequestContext(request, )
    return render(request, 'genes/genelist_create.html', {'form': form})



class GenesetDetailView(DetailView):
    model = GeneList
    context_object_name = "geneset"

    def get_template_names(self):
        return ["genes/genelist_detail.html"]

    def get_object(self):
        #get objects
        return GeneList.objects.filter(id=self.kwargs['pk'])[0]


    def get_context_data(self, **kwargs):
        context = super(GenesetDetailView, self).get_context_data(**kwargs)
        return context

# class GenesetDeleteView(DeleteView):
#     def get_object(self, queryset=None):
#         """ Hook to ensure object is owned by request.user. """
#         obj = super(GenesetDeleteView, self).get_object()
#         if not obj.owner == self.request.user:
#             raise Http404
#         return obj

def delete_genelist(request,pk):
    u = GeneList.objects.filter(id=pk)[0]
    
    if u.user == request.user:
        u.delete()
    
    return redirect(reverse('genes_list'))


class ListGene(ListView):
    model = Gene      #implies -> queryset = models.Car.objects.all()
    paginate_by = 100  #and that's it !!
    context_object_name = "genes"
    
    def get_queryset(self, **kwargs):
        q = self.request.GET.get('q', '')
        return Gene.objects.filter(Q(name__icontains=q)|Q(symbol__icontains=q))

    def post(self, request, *args, **kwargs):
        #get list of diseases
        genes_ids = request.POST.getlist('genes')
        genes = Gene.objects.filter(id__in=genes_ids)
        #create list of genes
        gene_list = []
        for gene in genes:
            gene_list.append(gene.symbol)
        #redirect to filter analysis page
        return redirect('/filter_analysis/?gene_list=%s' % ("%2C".join(gene_list)))

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(ListGene, self).get_context_data(**kwargs)
        # Add in a QuerySet of all the books
        context['genelists'] = GeneList.objects.all().order_by('name')

        return context

class GeneDetail(DetailView):
    model = Gene
    


# @login_required
# def populate_termsfromgo(request):
#     p = obo_parser.GODag('data/gene_ontology.1_2.obo')
#     goterms = []
#     goterms_dict = []
#     goterms_parents_children = {}
    
#     #print p["GO:0022610"]
#     #print p["GO:0022610"].parents
#     #print p["GO:0022610"].children
    
#     for item in p:
#       if p[item].id not in goterms_dict:
# 	goterms_dict.append(p[item].id)
# 	goterm = GoTerm(goid=p[item].id,
# 		      name=p[item].name,
# 		      namespace=p[item].namespace,
# 		      level=p[item].level,
# 		      is_obsolete=p[item].is_obsolete,
# 		      alt_ids=p[item].alt_ids
# 	)
	
	
# 	goterms_parents_children[p[item].id] = {}
# 	#print p[item]
# 	goterms_parents_children[p[item].id]['parents'] = []
# 	#print 'parents'
# 	for parent in p[item].parents:
# 	  #print parent
# 	  goterms_parents_children[p[item].id]['parents'].append(parent.id)
	
	
# 	goterms_parents_children[p[item].id]['children'] = []
# 	#print 'children'
# 	for child in p[item].children:
# 	  #print child
# 	  goterms_parents_children[p[item].id]['children'].append(child.id)
	
# 	goterms.append(goterm)
	      
#     GoTerm.objects.bulk_create(goterms)
#     print 'finished inserting files'
#     goterms = GoTerm.objects.all()
#     goterms_dict = {}
#     for goterm in goterms:
#       goterms_dict[goterm.goid] = goterm
      
#     print 'finished creating dict'
    
#     with transaction.commit_on_success():
      
#       for goid in goterms_parents_children:
# 	#get goterm
# 	goterm = goterms_dict[goid]
# 	#for each parent
# 	for parentgoid in goterms_parents_children[goid]['parents']:
# 	  goterm.parents.add(goterms_dict[parentgoid])
	
	
# 	#add children
# 	for childrengoid in goterms_parents_children[goid]['children']:
# 	  goterm.children.add(goterms_dict[childrengoid])
	
	
      
#       #add parents
#       #add children
	
#     #open file with GO's
#     print 'finish filling Terms from Go'
#     messages.add_message(request, messages.INFO, "Finish filling Terms from Go")
#     return redirect('/databases/')


@login_required
def populate_fromgo(request):
    #open file with GO's
    gofile = open('data/go_gene_terms')
    
    genes = {}
    gos = {}
    relationships = {}
    
    count_line = 0
    count2 = 0
    for line in gofile:
      #get line
      count_line += 1
      count2 += 1
      if count2 == 5000:
        print(count_line)
        count2 = 0
      line = line.split('\t')
      #GO
      go =  line[0]
      domain = line[1]
      category_name = line[2]
      definition = "\"".join(line[3].split('"')[1:-1])
      #Gene
      gene_symbol = line[4].upper()
      gene_description = line[5]
      
      if gene_symbol not in genes:
        gene = Gene(symbol=gene_symbol, description = gene_description)
        genes[gene_symbol] = gene
      else:
        gene = genes[gene_symbol] 

      if go not in gos:
        genecategory = GeneCategory(go=go, name=category_name, domain=domain)
        gos[go] = genecategory
      else:
        genecategory = gos[go]
      
      #add relationships
      if go not in relationships:
        relationships[go] = []
      if gene_symbol not in relationships[go]:
        relationships[go].append(gene_symbol)
      
    
    Gene.objects.bulk_create(list(genes.values()))
    GeneCategory.objects.bulk_create(list(gos.values()))
    print('Creating relationships')
    
    genes = Gene.objects.all()
    genes_dict = {}
    for gene in genes:
      genes_dict[gene.symbol] = gene
    
    
    gene_categories = GeneCategory.objects.all()
    categories_dict = {}
    for category in gene_categories:
      categories_dict[category.go] = category
    print('finished creating dicts')
    
    #RelationShips
    memberships = []
    count_membership = 0
    count_membership2 = 0
    for go in relationships:
      #gene_category = GeneCategory.objects.get(go=go)
      
      for gene_symbol in relationships[go]:
        count_membership += 1
        count_membership2 += 1
    if count_membership2 == 5000:
      print(count_membership)
      Membership.objects.bulk_create(memberships)
      memberships = []
      count_membership2 = 0
      
    #get gene
    #gene = Gene.objects.get(symbol=gene_symbol)
    membership = Membership(gene=genes_dict[gene_symbol], group=categories_dict[go])
    memberships.append(membership)
    #relationship_list.append(relationship)
    
    Membership.objects.bulk_create(memberships)
    print('finish filling genes from Go')
    messages.add_message(request, messages.INFO, "Finished filling genes from GO")
    return redirect('/databases/')
    
    
# Create your views here.
@login_required
def populate(request):
    
    hgnc = codecs.open('data/hgnc/hgnc_downloads.cgi.txt', 'r', "ISO-8859-1")
    header = hgnc.readline().strip().split('\t')
    count_gene = 1
    
    genes = []
    countline = 0
    for line in hgnc:
#        print count_gene

        count_gene += 1
        countline += 1
        #print countline
        if count_gene == 5000:
            print(countline)
            #bulk_insert(genes, show_sql=False)
            Gene.objects.bulk_create(genes)
            count_gene = 0
            genes = []
        
#        print line
        line = line.strip().split('\t')
#        print len(line)
#        print line
        #ugly hack
        if len(line) != 41:
            count = 41 - len(line) 
            for item in range(0,count):
                line.append('')
        
        gene = Gene(symbol = line[1],
                    name = line[2],
                    locus_type = line[4],
                    locus_group = line[5],
                    previous_symbols = line[6],
                    previous_names = line[7],
                    synonyms = line[8],
                    name_synonyms = line[9],
                    chromossome = line[10],
                    date_approved = line[11],
                    date_modified = line[12],
                    date_symbol_changes = line[13],
                    date_name_changes = line[14],
                    accesion_numbers = line[15],
                    enzyme_ids = line[16],
                    entrez_gene_id = line[17],
                    emsembl_gene_id = line[18],
                    mouse_genome_database_id = line[19],
                    specialist_database_links = line[20],
                    specialist_database_ids = line[21],
                    pubmed_ids = line[22],
                    refseq_ids = line[23],
                    gene_famili_tag = line[24],
                    gene_family_description = line[25],
                    record_type = line[26],
                    primary_ids = line[27],
                    secondary_ids = line[28],
                    ccds_ids = line[29],
                    vega_ids = line[30],
                    locus_specific_databases = line[31],
                    gdb_id_mapped = line[32],
                    entrez_gene_id_mapped = line[33],
                    omim_id = line[34],
                    refseq_id = line[35],
                    uniprot_id = line[36],
                    ensembl_id = line[37],
                    ucsc_id = line[38],
                    mouse_genome_database = line[39],
                    rat_genome_database = line[40])
    
        
        genes.append(gene)

    Gene.objects.bulk_create(genes)
    #bulk_insert(genes, show_sql=False)
    hgnc.close()
    print('finish filling genes')
    messages.add_message(request, messages.INFO, "Finished filling genes")
    return redirect('/databases/')


@login_required
def view(request, gene_name):
#    gene = Variant.objects.filter(gene_name=gene_name).annotate(num_individuals=Count('individuals'))
    gene_object = Gene.objects.get(symbol=gene_name)
    
    #get all variants present in this gene
    gene_variants = Variant.objects.filter(gene=gene_name)
    
    #variants by individuals
    variants_by_individuals = gene_variants.values('individual').annotate(total=Count('individual')).order_by('individual')
    
    #put all summary in this variable
    variants_summary = {}
    variants_summary['individual'] = {'known':0, 'novel':0, 'total':0} 
    for individual in variants_by_individuals:
        individual['individual'] = get_object_or_404(Individual, pk=individual['individual'])
        individual['novel'] = Variant.objects.filter(gene=gene_name, individual=individual['individual'], variant_id='.').count()
        individual['known'] = individual['total'] - individual['novel']
        
        #update total summary
        variants_summary['individual']['known'] += individual['known'] 
        variants_summary['individual']['novel'] += individual['novel']
        variants_summary['individual']['total'] += individual['total']
    
    
    
    #by effect and by individual
    #this is summary for snp_eff already (not by individual, only snp_eff)
    dna_variation = gene_variants.values('snpeff_effect').annotate(total=Count('snpeff_effect')).order_by('snpeff_effect')
    dna_variation_classes = dna_variation.values_list('snpeff_effect', flat=True)
    individuals_dna_variation = {}
    #for each individual
    for individual in variants_by_individuals:
        individual_dna_variation = {}
        #get all variants from individual by effect
        variants = Variant.objects.filter(gene=gene_name, individual=individual['individual']).values('snpeff_effect').annotate(total=Count('snpeff_effect')).order_by('snpeff_effect')
        for item in variants:
            individual_dna_variation[item['snpeff_effect']] = item['total'] 
        individuals_dna_variation[individual['individual']] = individual_dna_variation
    #print individuals_dna_variation
    
    
    functional_class = Variant.objects.filter(gene=gene_name).values('snpeff_func_class').annotate(total=Count('snpeff_func_class')).order_by('snpeff_func_class')
    
    functional_class_classes = functional_class.values_list('snpeff_func_class', flat=True)
    
    individuals_functional_class = {}
    for individual in variants_by_individuals:
        individual_functional_class = {}
        variants = Variant.objects.filter(gene=gene_name, individual=individual['individual']).values('snpeff_func_class').annotate(total=Count('snpeff_func_class')).order_by('snpeff_func_class')
        for item in variants:
            individual_functional_class[item['snpeff_func_class']] = item['total'] 
        individuals_functional_class[individual['individual']] = individual_functional_class
        
    
    impact = Variant.objects.filter(gene=gene_name).values('snpeff_impact').annotate(total=Count('snpeff_impact')).order_by('snpeff_impact')
    print(impact) 
    impact_classes = impact.values_list('snpeff_impact', flat=True)
    individuals_impact = {}
    for individual in variants_by_individuals:
        individual_impact = {}
        variants = Variant.objects.filter(gene=gene_name, individual=individual['individual']).values('snpeff_impact').annotate(total=Count('snpeff_impact')).order_by('snpeff_impact')
        for item in variants:
            individual_impact[item['snpeff_impact']] = item['total'] 
        individuals_impact[individual['individual']] = individual_impact
#    dna_variation = Variant.objects.filter(gene_name=gene_name).values_list('snp_eff').annotate(total=Count('snp_eff')).order_by('snp_eff')
#    functional_class = Variant.objects.filter(gene_name=gene_name).values_list('snp_eff_functional_class').annotate(total=Count('snp_eff_functional_class')).order_by('snp_eff_functional_class')
#    protein_impact = Variant.objects.filter(gene_name=gene_name).values_list('impact').annotate(total=Count('impact')).order_by('impact')
#    print gene.count()
    
    return render(request, 'genes/view.html', {'gene_object':gene_object,
                                                  'variants_by_individuals': variants_by_individuals,
                                                  'variants_summary':variants_summary,
                                                  'dna_variation':dna_variation, 
                                                  'dna_variation_classes':dna_variation_classes,
                                                  'individuals_dna_variation':individuals_dna_variation,
                                                  'functional_class':functional_class,
                                                  'functional_class_classes':functional_class_classes,
                                                  'individuals_functional_class':individuals_functional_class,
                                                  'impact':impact,
                                                  'impact_classes':impact_classes,
                                                  'individuals_impact':individuals_impact})



@login_required
def creategroup(request):
    if request.method == 'POST':
        form = GeneGroupForm(request.POST)        
        if form.is_valid():
            
            #use id for unique names
            genegroup = GeneGroup.objects.create(
                                                 name=form.cleaned_data['name'], 
                                                 genes=form.cleaned_data['genes'])
            genegroup.save()
            return HttpResponseRedirect('/filteranalysis/')
    else:
        form = GeneGroupForm()
    return render(request, 'genes/creategenegroup.html', {'form': form})

    
@login_required
def geneontology(request, go_id=''):
    #print go_id
    #if go_id != '':
      #goterms = GoTerm.objects.get(goid=go_id)
      ##GO:0003674 # molecular function
      ##GO:0005575 # celular component
      ##GO:0008150 # biological process
    
    go_ids = ['GO:0003674', 'GO:0005575', 'GO:0008150']
    goterms = GoTerm.objects.filter(goid__in = go_ids).order_by('name')
    
    return render(request, 'genes/geneontology.html', {'goterms':goterms})

    
def get_all_children(goterm):
    yield goterm.goid
#    lista = [goterm.goid]
    if goterm.children.all() != []:
        for child in goterm.children.all():
            yield child.goid
#            lista = lista + get_all_children(child)
#    return lista
    
  
    
@login_required
def geneontology_view(request, go_id=''):
#    print go_id
    if go_id != '':
      goterm = GoTerm.objects.get(goid=go_id)
      children = goterm.children.order_by('name')
      parents = goterm.parents.order_by('name')
    #get all children
    #print children
    
    all_children = [x for x in get_all_children(goterm)]
#    print all_children
#    
##    print all_children
##    for goterm_child in all_children:
##        print goterm_child 
#    
#    #test = GoTerm.objects.filter(id=goterm.id).prefetch_related('children')
#    #print test
    genes_categories = GeneCategory.objects.filter(go__in=all_children).values_list('id', flat=True)
    genes = Membership.objects.filter(group__in=genes_categories)
#    
#    genes = [] 
    
    paginator = Paginator(genes, 100) # Show 25 contacts per page
    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1
    try:
        genes = paginator.page(page)
    except PageNotAnInteger:
        # If page is not an integer, deliver first page.
        genes = paginator.page(1)
    except EmptyPage:
        # If page is out of range (e.g. 9999), deliver last page of results.
        genes = paginator.page(paginator.num_pages)
	
      
      
      
    print(len(genes))  
    
    
    return render(request, 'genes/geneontology_view.html', {'goterm':goterm, 'children':children, 'parents':parents, 'genes':genes})


@staff_member_required
def populate_cgd(request):
    CDGfile = open('data/CGD/CGD.txt')
    header = next(CDGfile)
    for line in CDGfile:
        gene_item = line.split('\t')
        # print gene_item
        cgd = CGDEntry()

        cgd.GENE = gene_item[0]
        cgd.ENTREZ_GENE_ID = gene_item[1]
        
        cgd.INHERITANCE = gene_item[3]
        cgd.AGE_GROUP = gene_item[4]
        cgd.ALLELIC_CONDITIONS = gene_item[5]

        cgd.COMMENTS = gene_item[8]
        cgd.INTERVENTION_RATIONALE = gene_item[9]
        
        references_list =[]
        references = gene_item[10].split(';')
        for ref in references:
            references_list.append(ref.strip())

        cgd.REFERENCES = references_list

        cgd.save()
        
        manifestations = gene_item[6].split(';')
        for manifestation in  manifestations:
            #try get manifestation
            manifestation = manifestation.strip()
            if manifestation != '':
                man_obj = Manifestation.objects.get_or_create(name=manifestation)[0]
                cgd.MANIFESTATION_CATEGORIES.add(man_obj)
                # cgd.MANIFESTATION_CATEGORIES.append(man_obj)  
                

        interventions = gene_item[7].split(';')
        for intervention in  interventions:
            #try get manifestation
            intervention = intervention.strip()
            int_obj = Intervention.objects.get_or_create(name=intervention)[0]
            cgd.INTERVENTION_CATEGORIES.add(int_obj)
            # cgd.INTERVENTION_CATEGORIES.append(int_obj)

        conditions = gene_item[2].split(';')
        for condition in conditions:
            condition = condition.strip()
            if condition != '':
                cond_obj = CGDCondition.objects.get_or_create(name=condition)[0]
                cgd.CONDITIONS.add(cond_obj)
                # cgd.CONDITIONS.append(cond_obj)

        

    return redirect('/genes')


@staff_member_required
def create_cgd_categories(request):
    #get categories
    manifestations = Manifestation.objects.all()
    recessive = open('%s/data/lupski/Lupski genes recessivos.txt' % (settings.BASE_DIR))
    recessive_gene_list = []
    for line in recessive:
        genes = line.split(',')
        for gene in genes:
            recessive_gene_list.append(gene.strip().upper())
    
    genelist = GeneList()
    genelist.name = 'Lupski recessive genes'
    genelist.genes = ",".join(recessive_gene_list)
    genelist.user=request.user
    genelist.save()

    dominant = open('%s/data/lupski/Lupski genes dominantes.txt' % (settings.BASE_DIR))
    dominant_gene_list = []
    for line in dominant:
        genes = line.split(',')
        for gene in genes:
            dominant_gene_list.append(gene.strip().upper())

    genelist = GeneList()
    genelist.name = 'Lupski dominant genes'
    genelist.genes = ",".join(recessive_gene_list)
    genelist.user=request.user
    genelist.save()

    xlinked = Disease.objects.filter(name__icontains="X-Linked")
    # print xlinked
    xlinked_gene_list = []
    for disease in xlinked:
        genes = disease.gene_names.split(',')
        for gene in genes:
            xlinked_gene_list.append(gene.strip())
    # xlinked_gene_list

    for manifestation in manifestations:
        
        # print manifestation

        #get genes from this manifestation
        cgdentries = CGDEntry.objects.filter(MANIFESTATION_CATEGORIES__in=[manifestation])
        gene_list = []
        for gene in cgdentries:
           gene_list.append(gene.GENE)

        #intersect with recessive
        intersection_lists = []
        intersection_lists.append(set(recessive_gene_list))
        intersection_lists.append(set(gene_list))
        genes_in_common_list = set.intersection(*intersection_lists)
        
        #create gene list
        genelist = GeneList()
        genelist.name = '%s_manifestation_recessive' % (manifestation)
        genelist.genes = ",".join(genes_in_common_list)
        genelist.user=request.user
        genelist.save()

        #intersect with dominant
        intersection_lists = []
        intersection_lists.append(set(dominant_gene_list))
        intersection_lists.append(set(gene_list))
        genes_in_common_list = set.intersection(*intersection_lists)
        
        #create gene list
        genelist = GeneList()
        genelist.name = '%s_manifestation_dominant' % (manifestation)
        genelist.genes = ",".join(genes_in_common_list)
        genelist.user=request.user
        genelist.save()

        #intersect with xlinked
        intersection_lists = []
        intersection_lists.append(set(xlinked_gene_list))
        intersection_lists.append(set(gene_list))
        genes_in_common_list = set.intersection(*intersection_lists)
        
        #create gene list
        genelist = GeneList()
        genelist.name = '%s_manifestation_xlinked' % (manifestation)
        genelist.genes = ",".join(genes_in_common_list)
        genelist.user=request.user
        genelist.save()

    return redirect(reverse('databases_index'))




from django import template

register = template.Library()


@register.filter
def paginator(context, adjacent_pages=2):
    current_page = context['page_obj'].number
    number_of_pages = context['paginator'].num_pages
    page_obj = context['page_obj']
    paginator = context['paginator']
    startPage = max(current_page - adjacent_pages, 1)
    endPage = current_page + adjacent_pages + 1
    if endPage > number_of_pages: endPage = number_of_pages + 1
    page_numbers = [n for n in range(startPage, endPage) \
        if 0 < n <= number_of_pages]

    return {
        'page_obj': page_obj,
        'paginator': paginator,
        'page': current_page,
        'pages': number_of_pages,
        'page_numbers': page_numbers,
        'has_previous': page_obj.has_previous(),
        'has_next': page_obj.has_next(),
        'show_first': 1 != current_page,
        'show_last': number_of_pages != current_page,
    }

register.inclusion_tag('list.html', takes_context=True)(paginator)


# Generated by Django 2.1.4 on 2018-12-27 08:50

from django.conf import settings
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion
import files.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=600)),
                ('size', models.BigIntegerField(blank=True, null=True)),
                ('file_type', models.TextField(blank=True, null=True)),
                ('extension', models.TextField(blank=True, null=True)),
                ('location', models.TextField(blank=True, null=True)),
                ('local_file', models.FileField(blank=True, help_text='File Format: VCF', max_length=600, upload_to=files.models.File.get_upload_path)),
                ('remote_location', models.TextField(blank=True, null=True)),
                ('url', models.TextField(blank=True, null=True)),
                ('status', models.TextField(blank=True, null=True)),
                ('last_output', models.TextField(blank=True, null=True)),
                ('params', django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True)),
                ('md5', models.TextField(blank=True, null=True)),
                ('last_modified', models.DateTimeField(blank=True, null=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, null=True)),
                ('modified_date', models.DateTimeField(auto_now=True, null=True)),
                ('user', models.ForeignKey(editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from subprocess import run, check_output
import os
import json

class B2():
    def __init__(self):
        pass
    def main(self):
        print('main')

    def install(self):
        command = '''
        git clone https://github.com/Backblaze/B2_Command_Line_Tool.git
        cd B2_Command_Line_Tool
        python setup.py build
        python setup.py install
        b2 authorize-account
        '''
        run(command,shell=True)

    def upload(self, source, dest):
        
        command = 'b2 upload-file mendelmd {} {}'.format(source,dest)
        
        output = check_output(command, shell=True).decode('utf-8')
        output = output.splitlines()
        print(output)
        results = ''.join(output[2:])
        print('results', results)
        results = json.loads(results)
        results[output[0].split(':',1)[0]] = output[0].split(':',1)[1:] 
        results[output[1].split(':',1)[0]] = output[1].split(':',1)[1:] 
        
        return(results)      

        # {
        # "action": "upload",
        # "fileId": "4_z9c8c149aa77346e564160516_f109f885ab1209044_d20180131_m235956_c000_v0001058_t0036",
        # "fileName": "test",
        # "size": 0,
        # "uploadTimestamp": 1517443196000
        # }

    def download(self, bucket, file):

        basename = os.path.basename(file)
        command = 'b2 download-file-by-name mendelmd {} {}'.format(file, file)
        run(command, shell=True)


from django.conf.urls import *
from django.views.generic import TemplateView

from cases.views import CaseDeleteView

from . import views


urlpatterns = [

    url(r'^$', views.cases_list, name='cases_list'),
    url(r'^create/$', views.create_case, name='create_case'),
    url(r'^delete/(?P<pk>\d+)$', CaseDeleteView.as_view(), {}, 'delete_case'),
    url(r'^edit/(?P<case_id>\d+)/$', views.edit, name='edit_case'),
    url(r'^view/(?P<case_id>\d+)/$', views.view_case, name='view_case'),
    url(r'^view/(?P<case_id>\d+)/$', views.view_case, name='view_case'),
    url(r'^analyze/(?P<case_id>\d+)/(?P<analysis>\w+)/(?P<inheritance>\w+)/$', views.analysis, name='analyze_case'),
    

    # url(r'^new/$', IndividualCreateView.as_view(), {}, 'individual-new'),
    # url(r'^delete/(?P<pk>\d+)$', IndividualDeleteView.as_view(), {}, 'individual-delete'),
    # url(r'^edit/(?P<individual_id>\d+)/$', 'edit', name='individual_edit'),
    
    # url(r'^success/$', TemplateView.as_view(template_name='individuals/sucess.html'), name="sucess"),

    # url(r'^view/(?P<individual_id>\d+)/$', 'view', name='individual_view'),
    
    # url(r'^view/(?P<individual_id>\d+)/medical_conditions/$', 'view_medical_conditions', name='individual_view_medical_conditions'),
    # url(r'^view/(?P<individual_id>\d+)/medicines/$', 'view_medicines', name='individual_view_medicines'),
    # url(r'^view/(?P<individual_id>\d+)/hgmd/$', 'view_hgmd_mutations', name='view_hgmd_mutations'),
    


    # url(r'^browse/(?P<individual_id>\d+)/$', 'browse', name='individual_browse'),
    # url(r'^export_csv/(?P<individual_id>\d+)/$', 'export_csv', name='export_csv'),
    # url(r'^populate/(?P<individual_id>\d+)/$', 'populate', name='individual_populate'),
    # url(r'^populate_mongo/(?P<individual_id>\d+)/$', 'populate_mongo', name='individual_populate_mongo'),
    
    # url(r'^annotate/(?P<individual_id>\d+)/$', 'annotate', name='individual_annotate'),
    # url(r'^download/(?P<individual_id>\d+)/$', 'download', name='individual_download'),
    # url(r'^download_annotated/(?P<individual_id>\d+)/$', 'download_annotated', name='individual_download_annotated'),
    # url(r'^download_annovar/(?P<individual_id>\d+)/$', 'download_annovar', name='individual_download_annovar'),
    # url(r'^download_csv/(?P<individual_id>\d+)/$', 'download_csv', name='individual_download_csv'),
    # url(r'^find_individual_diseases/(?P<individual_id>\d+)/$', 'find_individual_diseases', name='find_individual_diseases'),
    # url(r'^find_individual_medicines/(?P<individual_id>\d+)/$', 'find_individual_medicines', name='find_individual_medicines'),
    # url(r'^create_group/$', 'create_group', name='create_group'),
    # url(r'^delete_group/(?P<pk>\d+)$', GroupDeleteView.as_view(), {}, 'group_delete'),
    # #url(r'^delete_group/(?P<group_id>\d+)/$', 'delete_group', name='delete_group'),
    # url(r'^comparison/$', 'comparison', name='comparison'),
    # url(r'^export_comparison/$', 'export_comparison', name='export_comparison'),   
]


from django.db import models
from django.contrib.postgres.fields import JSONField
from django.contrib.auth.models import User

# Create your models here.

class App(models.Model):
    def get_absolute_url(self):
        return "/apps/"

    user = models.ForeignKey(User, on_delete=models.CASCADE, editable=False)
    name = models.CharField(max_length=255)
    status = models.CharField(max_length=255)
    category = models.CharField(max_length=255)
    source = models.CharField(max_length=600, null=True, blank=True)
    repository = models.CharField(max_length=600, null=True, blank=True)
    install = models.CharField(max_length=600, null=True, blank=True)
    main = models.CharField(max_length=600, null=True, blank=True)
    type = models.CharField(max_length=255)
    config = JSONField(null=True, blank=True)

    def __str__(self):
        return self.name



from django.conf import settings

from django.conf.urls import *

from django.contrib import admin
admin.autodiscover()

from . import views

urlpatterns = [
    url(r"^$", views.index, name="databases_index"),
    url(r"^dbnfsp/$", views.dbnfsp_index, name="dbnfsp_index"),
    url(r"^1000genomes/$", views.genomes1k_index, name="1000genomes_index"),
]


from django.forms import ModelForm
from django import forms
from .models import File

class UploadForm(ModelForm):

    vcf_file = forms.CharField(max_length=600, required=False)

    class Meta:
        model = File
        fields = '__all__'

# class FileForm(forms.Form):
#     OPTIONS = (
#                 ("fastq", "fastq"),
#                 ("bam", "bam"),
#                 ("vcf", "vcf"),
#                 )
    # extension = forms.MultipleChoiceField(choices=OPTIONS, required=False)



from django.conf import settings
from django.conf.urls import *

from django.views.generic import DetailView, ListView, CreateView, UpdateView, DeleteView
from django.contrib import admin
from diseases.views import DiseaseListView
from diseases.models import Disease

admin.autodiscover()

from . import views

from django.contrib.auth.decorators import login_required, permission_required


urlpatterns = [
    url(r"^populate/$", views.populate, name="diseases_populate"),
    url(r"^populate_genes/$", views.populate_genes, name="populate_genes"),

    url(r"^populate_hgmd_genes/$", views.populate_hgmd_genes, name="populate_hgmd_genes"),
    url(r"^populate_hgmd_mutations/$", views.populate_hgmd_mutations, name="populate_hgmd_mutations"),
    
    #url(r"^$", 'diseases.views.list', name="diseases_list"),
    
    url(r'^$', login_required(DiseaseListView.as_view(model=Disease)), name="diseases_list"),
    

    
    url(r'^view/(?P<disease_id>\d+)/$', views.view, name='disease_view'),
    ]